
from gaia_sdk.graphql.request.type.BehaviourExecution import BehaviourExecution
from gaia_sdk.graphql.request.type.SkillProvisionBuildJob import SkillProvisionBuildJob
from gaia_sdk.graphql.request.type.IdentityMetrics import IdentityMetrics
from gaia_sdk.graphql.request.type.BehaviourNodeExecution import BehaviourNodeExecution
from gaia_sdk.graphql.request.type.BehaviourExecutionDetail import BehaviourExecutionDetail

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Experience(list):
    """
    Container type for runtime information
    """

    def behaviour_execution(self, identityId: str, processInstanceId: str, config: Callable[['BehaviourExecutionDetail'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("identityId", identityId)
            name2 = registry.register("processInstanceId", processInstanceId)
            entity = BehaviourExecutionDetail()
            config(entity)
            return f'behaviourExecution(identityId:{name1}, processInstanceId:{name2})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def behaviour_executions(self, identityId: str, limit: int, offset: int, startDate: str, endDate: str, config: Callable[['BehaviourExecution'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("identityId", identityId)
            name2 = registry.register("limit", limit)
            name3 = registry.register("offset", offset)
            name4 = registry.register("startDate", startDate)
            name5 = registry.register("endDate", endDate)
            entity = BehaviourExecution()
            config(entity)
            return f'behaviourExecutions(identityId:{name1}, limit:{name2}, offset:{name3}, startDate:{name4}, endDate:{name5})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def behaviour_node_executions(self, config: Callable[['BehaviourNodeExecution'], None]):
        def callback(registry: VariableRegistry):
            entity = BehaviourNodeExecution()
            config(entity)
            return "behaviour_node_executions {" + entity.render(registry) + "}"
        self.append(callback)

    def identity_metrics(self, identityId: str, startDate: str, endDate: str, limit: int, config: Callable[['IdentityMetrics'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("identityId", identityId)
            name2 = registry.register("startDate", startDate)
            name3 = registry.register("endDate", endDate)
            name4 = registry.register("limit", limit)
            entity = IdentityMetrics()
            config(entity)
            return f'identityMetrics(identityId:{name1}, startDate:{name2}, endDate:{name3}, limit:{name4})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def skill_provision_build_jobs(self, tenantId: str, config: Callable[['SkillProvisionBuildJob'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("tenantId", tenantId)
            entity = SkillProvisionBuildJob()
            config(entity)
            return f'skillProvisionBuildJobs(tenantId:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.request.type.ConnectNodeKnowledge import ConnectNodeKnowledge

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class ConnectKnowledge(list):

    def node(self, nodeId: str, config: Callable[['ConnectNodeKnowledge'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("nodeId", nodeId)
            entity = ConnectNodeKnowledge()
            config(entity)
            return f'node(nodeId:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



import json

import rx
import rx.operators as ops
from rx.core.typing import Observable, Scheduler

from gaia_sdk.http import GaiaStreamClient


class SkillProvisionCanceledResponse:
    def __init__(self, reference: str):
        self._reference = reference

    @property
    def reference(self) -> str:
        return self._reference


class SkillBuildResponse:
    def __init__(self, reference):
        self._reference = reference

    @property
    def reference(self) -> str:
        return self._reference


class SkillEvaluation:
    def __init__(self, response: dict):
        self._response = response

    @property
    def response(self):
        return self._response


class SkillRef:
    def __init__(self, uri: str, client: GaiaStreamClient, scheduler: Scheduler):
        self._uri = uri
        self._client = client
        self._scheduler = scheduler

    def start(self) -> Observable[any]:
        return rx.from_callable(lambda: self._client.post_json({'uri': self._uri}, url_postfix="/skill/start"),
                                self._scheduler) \
            .pipe(
            ops.map(lambda r: {})
        )

    def stop(self) -> Observable[any]:
        return rx.from_callable(lambda: self._client.post_json({'uri': self._uri}, url_postfix="/skill/stop"),
                                self._scheduler) \
            .pipe(
            ops.map(lambda r: {})
        )

    def logs(self, number_of_lines: int) -> Observable[str]:
        return rx.from_callable(lambda: self._client.post_json({'uri': self._uri, 'numberOfLines': number_of_lines},
                                                               url_postfix="/skill/logs"), self._scheduler) \
            .pipe(
            ops.map(lambda r: json.loads(r.content)),
            ops.flat_map(lambda r: rx.from_iterable(r['logLines']))
        )

    def cancel(self) -> Observable[SkillProvisionCanceledResponse]:
        return rx.from_callable(lambda: self._client.post_json({'uri': self._uri}, url_postfix="/skill/cancel"),
                                self._scheduler) \
            .pipe(
            ops.map(lambda r: json.loads(r.content)),
            ops.map(lambda r: SkillProvisionCanceledResponse(r["reference"]))
        )

    def build(self) -> Observable[SkillBuildResponse]:
        return rx.from_callable(lambda: self._client.post_json({'uri': self._uri}, url_postfix="/skill/build"),
                                self._scheduler) \
            .pipe(
            ops.map(lambda r: json.loads(r.content)),
            ops.map(lambda r: SkillBuildResponse(r["reference"]))
        )

    def evaluate(self, payload: dict, contract: str = None) -> Observable[SkillEvaluation]:
        request = {'uri': self._uri, 'payload': payload}
        if contract is not None:
            request['contract'] = contract
        return rx.from_callable(
            lambda: self._client.post_json(request, url_postfix="/skill/evaluate"),
            self._scheduler) \
            .pipe(
            ops.map(lambda response: json.loads(response.content)),
            ops.map(lambda d: SkillEvaluation(d)))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class UpdateTenantImpulse():
    """
    The specification to update a tenant
    """
    tenantId: str
    qualifier: str
    implicitIdentities: List[str]
    explicitIdentities: List[str]

    def __init__(self, tenantId: str, qualifier: str, implicitIdentities: List[str], explicitIdentities: List[str]):
        self.tenantId = tenantId
        self.qualifier = qualifier
        self.implicitIdentities = implicitIdentities
        self.explicitIdentities = explicitIdentities

    def __eq__(self, other):
        if type(other) is type(self):
            return self.tenantId == other.tenantId and self.qualifier == other.qualifier and self.implicitIdentities == other.implicitIdentities and self.explicitIdentities == other.explicitIdentities
        return False

    def __repr__(self):
        return {'tenantId': self.tenantId, 'qualifier': self.qualifier, 'implicitIdentities': self.implicitIdentities, 'explicitIdentities': self.explicitIdentities}




from gaia_sdk.graphql.request.type.OnConversed import OnConversed

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Interaction(list):

    def on_conversed(self, config: Callable[['OnConversed'], None]):
        def callback(registry: VariableRegistry):
            entity = OnConversed()
            config(entity)
            return "on_conversed {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.request.type.UpdatedUserImpulse import UpdatedUserImpulse
from gaia_sdk.graphql.request.type.UpdatedTenantImpulse import UpdatedTenantImpulse
from gaia_sdk.graphql.request.type.UpdatedBehaviourImpulse import UpdatedBehaviourImpulse
from gaia_sdk.graphql.request.type.UpdatedApiKeyImpulse import UpdatedApiKeyImpulse
from gaia_sdk.graphql.request.type.UpdatedSkillImpulse import UpdatedSkillImpulse
from gaia_sdk.graphql.request.type.UpdatedCodeImpulse import UpdatedCodeImpulse
from gaia_sdk.graphql.request.type.UpdatedRoleImpulse import UpdatedRoleImpulse
from gaia_sdk.graphql.request.type.UpdatedIntentImpulse import UpdatedIntentImpulse
from gaia_sdk.graphql.request.type.UpdatedStatementImpulse import UpdatedStatementImpulse
from gaia_sdk.graphql.request.type.UpdatedFulfilmentImpulse import UpdatedFulfilmentImpulse
from gaia_sdk.graphql.request.type.UpdatedSkillProvisionImpulse import UpdatedSkillProvisionImpulse
from gaia_sdk.graphql.request.type.UpdatedIdentityImpulse import UpdatedIdentityImpulse
from gaia_sdk.graphql.request.type.UpdatedPromptImpulse import UpdatedPromptImpulse
from gaia_sdk.graphql.request.input.UpdateBehaviourImpulse import UpdateBehaviourImpulse
from gaia_sdk.graphql.request.input.UpdateSkillProvisionImpulse import UpdateSkillProvisionImpulse
from gaia_sdk.graphql.request.input.UpdateUserImpulse import UpdateUserImpulse
from gaia_sdk.graphql.request.input.UpdateIntentImpulse import UpdateIntentImpulse
from gaia_sdk.graphql.request.input.UpdateCodeImpulse import UpdateCodeImpulse
from gaia_sdk.graphql.request.input.UpdatePromptImpulse import UpdatePromptImpulse
from gaia_sdk.graphql.request.input.UpdateSkillImpulse import UpdateSkillImpulse
from gaia_sdk.graphql.request.input.UpdateIdentityImpulse import UpdateIdentityImpulse
from gaia_sdk.graphql.request.input.UpdateStatementImpulse import UpdateStatementImpulse
from gaia_sdk.graphql.request.input.UpdateApiKeyImpulse import UpdateApiKeyImpulse
from gaia_sdk.graphql.request.input.UpdateRoleImpulse import UpdateRoleImpulse
from gaia_sdk.graphql.request.input.UpdateTenantImpulse import UpdateTenantImpulse
from gaia_sdk.graphql.request.input.UpdateFulfilmentImpulse import UpdateFulfilmentImpulse

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class UpdateKnowledge(list):

    """
    updates a list of identities with the given specifications
    """
    def identities(self, impulses: List[UpdateIdentityImpulse], config: Callable[['UpdatedIdentityImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedIdentityImpulse()
            config(entity)
            return f'identities(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of tenants with the given specifications
    """
    def tenants(self, impulses: List[UpdateTenantImpulse], config: Callable[['UpdatedTenantImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedTenantImpulse()
            config(entity)
            return f'tenants(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of users with the given specifications
    """
    def users(self, impulses: List[UpdateUserImpulse], config: Callable[['UpdatedUserImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedUserImpulse()
            config(entity)
            return f'users(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of api keys with the given specifications
    """
    def api_keys(self, impulses: List[UpdateApiKeyImpulse], config: Callable[['UpdatedApiKeyImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedApiKeyImpulse()
            config(entity)
            return f'apiKeys(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of roles with the given specifications
    """
    def roles(self, impulses: List[UpdateRoleImpulse], config: Callable[['UpdatedRoleImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedRoleImpulse()
            config(entity)
            return f'roles(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of intents with the given specifications
    """
    def intents(self, impulses: List[UpdateIntentImpulse], config: Callable[['UpdatedIntentImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedIntentImpulse()
            config(entity)
            return f'intents(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of prompts with the given specifications
    """
    def prompts(self, impulses: List[UpdatePromptImpulse], config: Callable[['UpdatedPromptImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedPromptImpulse()
            config(entity)
            return f'prompts(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of statements with the given specifications
    """
    def statements(self, impulses: List[UpdateStatementImpulse], config: Callable[['UpdatedStatementImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedStatementImpulse()
            config(entity)
            return f'statements(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of fulfilments with the given specifications
    """
    def fulfilments(self, impulses: List[UpdateFulfilmentImpulse], config: Callable[['UpdatedFulfilmentImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedFulfilmentImpulse()
            config(entity)
            return f'fulfilments(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of behaviours with the given specifications
    """
    def behaviours(self, impulses: List[UpdateBehaviourImpulse], config: Callable[['UpdatedBehaviourImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedBehaviourImpulse()
            config(entity)
            return f'behaviours(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of codes with the given specifications
    """
    def codes(self, impulses: List[UpdateCodeImpulse], config: Callable[['UpdatedCodeImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedCodeImpulse()
            config(entity)
            return f'codes(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of skills with the given specifications
    """
    def skills(self, impulses: List[UpdateSkillImpulse], config: Callable[['UpdatedSkillImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedSkillImpulse()
            config(entity)
            return f'skills(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    updates a list of skill provisions with the given specifications
    """
    def skill_provisions(self, impulses: List[UpdateSkillProvisionImpulse], config: Callable[['UpdatedSkillProvisionImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = UpdatedSkillProvisionImpulse()
            config(entity)
            return f'skillProvisions(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class PerceiveDataImpulse():
    """
    Input for data perception impulse.
    """
    identityId: str
    eventName: str
    eventData: dict

    def __init__(self, identityId: str, eventName: str, eventData: dict):
        self.identityId = identityId
        self.eventName = eventName
        self.eventData = eventData

    def __eq__(self, other):
        if type(other) is type(self):
            return self.identityId == other.identityId and self.eventName == other.eventName and self.eventData == other.eventData
        return False

    def __repr__(self):
        return {'identityId': self.identityId, 'eventName': self.eventName, 'eventData': self.eventData}



from gaia_sdk.api.VariableRegistry import VariableRegistry
import itertools
import random
from typing import List, Callable

from gaia_sdk.api.transporter.abstract_transporter import ITransporter
from gaia_sdk.graphql.GaiaResponse import QueryResponse
from gaia_sdk.graphql.request.type import Query
from gaia_sdk.graphql.GaiaResponse import MutationResponse
from gaia_sdk.graphql.request.type import Mutation
from gaia_sdk.api.client_options import ClientOptions
from gaia_sdk.http.request.Payload import Payload


class GaiaClient(object):

    transporter: ITransporter
    options: ClientOptions

    def __init__(self, transporter: ITransporter, options: ClientOptions):
        self.transporter = transporter
        self.options = options

    def __eq__(self, other):
        return self.options == other.options

    def __repr__(self):
        return {'options': self.options}

    def execute_native(self, statement: str, variables: dict):
        payload = {"statement": statement, "variables": variables}
        return self.transporter.transport(self.options, Payload.json(payload)).json()

    def query(self, request: Query) -> QueryResponse:
        statement, variables = self.get_statement("query", request)
        return self.execute_native(statement, variables)

    def mutation(self, request: Mutation) -> MutationResponse:
        statement, variables = self.get_statement("mutation", request)
        return self.execute_native(statement, variables)

    def get_statement(self, name: str, values: List[Callable[[VariableRegistry], str]]):
        registry = VariableRegistry()
        fields = " ".join(map(lambda x: x(registry), values))
        if len(registry.getDatatypes()) == 0:
            statement = f'{name} gaia ' + '{ ' + fields + ' }'
            return statement, registry.getVariables()

        statement = name + ' gaia(' + ", ".join(registry.getDatatypes()) + ')  {' + fields + ' }'
        return statement, registry.getVariables()



from gaia_sdk.graphql.response.type.Query import Query
from gaia_sdk.graphql.response.type.Mutation import Mutation
from gaia_sdk.graphql.response.type.Subscription import Subscription


class QueryResponse:
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        return self.dictionary == other.dictionary

    def __repr__(self):
        return {'dictionary': self.dictionary}

    @property
    def data(self) -> Query:
        return Query(self.dictionary.get("data"))

    @property
    def extensions(self) -> dict:
        return self.dictionary.get("extensions")

    @property
    def errors(self) -> list:
        return self.dictionary.get("errors")


class MutationResponse:
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        return self.dictionary == other.dictionary

    def __repr__(self):
        return {'dictionary': self.dictionary}

    @property
    def data(self) -> Mutation:
        return Mutation(self.dictionary.get("data"))

    @property
    def extensions(self) -> dict:
        return self.dictionary.get("extensions")

    @property
    def errors(self) -> list:
        return self.dictionary.get("errors")


class SubscriptionResponse:
    @property
    def data(self) -> Subscription:
        return self.data

    @property
    def extensions(self) -> dict:
        return self.extensions

    @property
    def errors(self) -> list:
        return self.errors




from typing import Callable, List, TypeVar

import rx
from rx import operators as ops, pipe
from rx.core.typing import Observable

from gaia_sdk.graphql import QueryRes, MutationRes
from gaia_sdk.graphql.GaiaResponse import QueryResponse, MutationResponse

T = TypeVar('T')


def mapQ(observable: Observable[QueryResponse], mapper: Callable[[QueryRes], T]) -> Observable[T]:
    def flat_map(obj: QueryResponse):
        if obj.errors and len(obj.errors) > 0:
            return rx.throw(obj.errors[0]["message"])
        return rx.of(mapper(obj.data))

    return pipe(ops.map(lambda x: QueryResponse(x)),
                ops.flat_map(flat_map))(observable)


def flat_mapQ(observable: Observable[QueryResponse], mapper: Callable[[QueryRes], List[T]]) -> Observable[T]:
    def flat_map(obj: QueryResponse):
        if obj.errors and len(obj.errors) > 0:
            return rx.throw(obj.errors[0]["message"])
        return rx.from_iterable(mapper(obj.data))

    return pipe(ops.map(lambda x: QueryResponse(x)),
                ops.flat_map(flat_map))(observable)


def mapM(observable: Observable[MutationResponse], mapper: Callable[[MutationRes], T]) -> Observable[T]:
    def flat_map(response: MutationResponse):
        if response.errors and len(response.errors) > 0:
            return rx.throw(response.errors[0]["message"])
        return rx.of(mapper(response.data))

    return pipe(ops.map(lambda x: MutationResponse(x)),
                ops.flat_map(flat_map))(observable)


def flat_mapM(observable: Observable[MutationResponse], mapper: Callable[[MutationRes], List[T]]) -> Observable[T]:
    def flat_map(obj: MutationResponse):
        if obj.errors and len(obj.errors) > 0:
            return rx.throw(obj.errors[0]["message"])
        return rx.from_iterable(mapper(obj.data))

    return pipe(
        ops.map(lambda x: MutationResponse(x)),
        ops.flat_map(flat_map)
    )(observable)





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class EdgeKeyOne(list):
    """
    This entity represents the output of an edge delete impulse
    """

    def source(self):
        self.append(lambda x: "source")

    def edge_id(self):
        self.append(lambda x: "edgeId")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



from gaia_sdk.api.GaiaCredentials import GaiaCredentials
from gaia_sdk.api.client_options import ClientOptions
from gaia_sdk.http.GaiaStreamClient import GaiaStreamClient
from gaia_sdk.http.HttpTransporter import HttpTransporter

class GaiaStreamClientFactory(object):
    def http(self, url):
        return GaiaHttpStreamClientBuilder(url)


class GaiaHttpStreamClientBuilder(object):

    def __init__(self, url):
        self.url = url

    def with_credentials(self, credentials: GaiaCredentials):
        self.credentials = credentials
        return self

    def build(self):
        return GaiaStreamClient(HttpTransporter(self.url), ClientOptions(self.credentials))



from gaia_sdk.api.crypto import HMAC
import base64
import json


class HMACTokenBuilder(object):

    def with_client_options(self, client_options):
        self.client_options = client_options
        return self

    def with_nonce(self, nonce):
        self.nonce = nonce
        return self

    def with_payload(self, payload):
        self.payload = payload
        return self

    def with_timestamp(self, timestamp):
        self.timestamp = timestamp
        return self

    def build(self):
        """
        Authorization: "HMAC-SHA512 " + API_KEY + "_" +
        base64(hmac-sha512( content, content_type, sensor_type, timestamp, nonce )) + "_" + timestamp + "_" + nonce
        """
        HTTP_SENSOR_TYPE = "http"
        sep = "_"

        if isinstance(self.payload, bytes):
            data_to_encode = self.payload
        else:
            data_to_encode = json.dumps(self.payload).encode("utf-8")

        base64_string = base64.b64encode(data_to_encode).decode()

        arrayToHash = [base64_string, self.client_options.content_type,
                       HTTP_SENSOR_TYPE, self.timestamp, self.nonce]
        prepareToHash = '_'.join([str(x) for x in arrayToHash])
        hmac = HMAC(self.client_options.credentials.apiSecret)

        signature = hmac.hash512(prepareToHash)
        token = "HMAC-SHA512 " + self.client_options.credentials.apiKey + sep + signature + sep + str(
            self.timestamp) + sep + self.nonce
        return token





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class CreateSkillImpulse():
    """
    The specification to create a skill instance
    """
    tenantId: str
    qualifier: str
    appendent: str
    labelList: List[str]
    repositoryUri: str

    def __init__(self, tenantId: str, qualifier: str, appendent: str, labelList: List[str], repositoryUri: str):
        self.tenantId = tenantId
        self.qualifier = qualifier
        self.appendent = appendent
        self.labelList = labelList
        self.repositoryUri = repositoryUri

    def __eq__(self, other):
        if type(other) is type(self):
            return self.tenantId == other.tenantId and self.qualifier == other.qualifier and self.appendent == other.appendent and self.labelList == other.labelList and self.repositoryUri == other.repositoryUri
        return False

    def __repr__(self):
        return {'tenantId': self.tenantId, 'qualifier': self.qualifier, 'appendent': self.appendent, 'labelList': self.labelList, 'repositoryUri': self.repositoryUri}



from typing import Dict
from typing import Any

class ClientOptions:
    """
    This class stores the credentials (JWT or HMAC-ApiKey&SecretKey tuple) used by the transporter implementation
    to establish a connection to G.A.I.A.
    """

    def __init__(self, credentials, content_type="application/json", request_parameters={}):
        self.credentials = credentials
        self.content_type = content_type
        self.request_parameters = request_parameters

    def with_content_type(self, content_type: str):
        return ClientOptions(self.credentials, content_type, self.request_parameters)

    def with_request_parameters(self, request_parameters: Dict[str, Any]):
        return ClientOptions(self.credentials, self.content_type, request_parameters)



class IdentitySourceRequestImpulse:
    id: str

    def __init__(self, identity_id: str):
        self.identity_id = identity_id

    def __eq__(self, other):
        return self.identity_id == other.identity_id

    def __repr__(self):
        return {'identityId': self.identity_id}




from gaia_sdk.graphql.request.type.Query import Query
from gaia_sdk.graphql.request.type.Mutation import Mutation
from gaia_sdk.graphql.request.type.Subscription import Subscription

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Schema(list):

    def query(self, config: Callable[['Query'], None]):
        def callback(registry: VariableRegistry):
            entity = Query()
            config(entity)
            return "query {" + entity.render(registry) + "}"
        self.append(callback)

    def mutation(self, config: Callable[['Mutation'], None]):
        def callback(registry: VariableRegistry):
            entity = Mutation()
            config(entity)
            return "mutation {" + entity.render(registry) + "}"
        self.append(callback)

    def subscription(self, config: Callable[['Subscription'], None]):
        def callback(registry: VariableRegistry):
            entity = Subscription()
            config(entity)
            return "subscription {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.response.type.UpdatedUserImpulse import UpdatedUserImpulse
from gaia_sdk.graphql.response.type.UpdatedTenantImpulse import UpdatedTenantImpulse
from gaia_sdk.graphql.response.type.UpdatedBehaviourImpulse import UpdatedBehaviourImpulse
from gaia_sdk.graphql.response.type.UpdatedApiKeyImpulse import UpdatedApiKeyImpulse
from gaia_sdk.graphql.response.type.UpdatedSkillImpulse import UpdatedSkillImpulse
from gaia_sdk.graphql.response.type.UpdatedCodeImpulse import UpdatedCodeImpulse
from gaia_sdk.graphql.response.type.UpdatedRoleImpulse import UpdatedRoleImpulse
from gaia_sdk.graphql.response.type.UpdatedIntentImpulse import UpdatedIntentImpulse
from gaia_sdk.graphql.response.type.UpdatedStatementImpulse import UpdatedStatementImpulse
from gaia_sdk.graphql.response.type.UpdatedFulfilmentImpulse import UpdatedFulfilmentImpulse
from gaia_sdk.graphql.response.type.UpdatedSkillProvisionImpulse import UpdatedSkillProvisionImpulse
from gaia_sdk.graphql.response.type.UpdatedIdentityImpulse import UpdatedIdentityImpulse
from gaia_sdk.graphql.response.type.UpdatedPromptImpulse import UpdatedPromptImpulse
from gaia_sdk.graphql.request.input.UpdateBehaviourImpulse import UpdateBehaviourImpulse
from gaia_sdk.graphql.request.input.UpdateSkillProvisionImpulse import UpdateSkillProvisionImpulse
from gaia_sdk.graphql.request.input.UpdateUserImpulse import UpdateUserImpulse
from gaia_sdk.graphql.request.input.UpdateIntentImpulse import UpdateIntentImpulse
from gaia_sdk.graphql.request.input.UpdateCodeImpulse import UpdateCodeImpulse
from gaia_sdk.graphql.request.input.UpdatePromptImpulse import UpdatePromptImpulse
from gaia_sdk.graphql.request.input.UpdateSkillImpulse import UpdateSkillImpulse
from gaia_sdk.graphql.request.input.UpdateIdentityImpulse import UpdateIdentityImpulse
from gaia_sdk.graphql.request.input.UpdateStatementImpulse import UpdateStatementImpulse
from gaia_sdk.graphql.request.input.UpdateApiKeyImpulse import UpdateApiKeyImpulse
from gaia_sdk.graphql.request.input.UpdateRoleImpulse import UpdateRoleImpulse
from gaia_sdk.graphql.request.input.UpdateTenantImpulse import UpdateTenantImpulse
from gaia_sdk.graphql.request.input.UpdateFulfilmentImpulse import UpdateFulfilmentImpulse

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class UpdateKnowledge:
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    updates a list of identities with the given specifications
    """
    @property
    def identities(self) -> List[UpdatedIdentityImpulse]:
        return list(map(lambda x: UpdatedIdentityImpulse(x), self.dictionary.get("identities")))
    """
    updates a list of tenants with the given specifications
    """
    @property
    def tenants(self) -> List[UpdatedTenantImpulse]:
        return list(map(lambda x: UpdatedTenantImpulse(x), self.dictionary.get("tenants")))
    """
    updates a list of users with the given specifications
    """
    @property
    def users(self) -> List[UpdatedUserImpulse]:
        return list(map(lambda x: UpdatedUserImpulse(x), self.dictionary.get("users")))
    """
    updates a list of api keys with the given specifications
    """
    @property
    def api_keys(self) -> List[UpdatedApiKeyImpulse]:
        return list(map(lambda x: UpdatedApiKeyImpulse(x), self.dictionary.get("apiKeys")))
    """
    updates a list of roles with the given specifications
    """
    @property
    def roles(self) -> List[UpdatedRoleImpulse]:
        return list(map(lambda x: UpdatedRoleImpulse(x), self.dictionary.get("roles")))
    """
    updates a list of intents with the given specifications
    """
    @property
    def intents(self) -> List[UpdatedIntentImpulse]:
        return list(map(lambda x: UpdatedIntentImpulse(x), self.dictionary.get("intents")))
    """
    updates a list of prompts with the given specifications
    """
    @property
    def prompts(self) -> List[UpdatedPromptImpulse]:
        return list(map(lambda x: UpdatedPromptImpulse(x), self.dictionary.get("prompts")))
    """
    updates a list of statements with the given specifications
    """
    @property
    def statements(self) -> List[UpdatedStatementImpulse]:
        return list(map(lambda x: UpdatedStatementImpulse(x), self.dictionary.get("statements")))
    """
    updates a list of fulfilments with the given specifications
    """
    @property
    def fulfilments(self) -> List[UpdatedFulfilmentImpulse]:
        return list(map(lambda x: UpdatedFulfilmentImpulse(x), self.dictionary.get("fulfilments")))
    """
    updates a list of behaviours with the given specifications
    """
    @property
    def behaviours(self) -> List[UpdatedBehaviourImpulse]:
        return list(map(lambda x: UpdatedBehaviourImpulse(x), self.dictionary.get("behaviours")))
    """
    updates a list of codes with the given specifications
    """
    @property
    def codes(self) -> List[UpdatedCodeImpulse]:
        return list(map(lambda x: UpdatedCodeImpulse(x), self.dictionary.get("codes")))
    """
    updates a list of skills with the given specifications
    """
    @property
    def skills(self) -> List[UpdatedSkillImpulse]:
        return list(map(lambda x: UpdatedSkillImpulse(x), self.dictionary.get("skills")))
    """
    updates a list of skill provisions with the given specifications
    """
    @property
    def skill_provisions(self) -> List[UpdatedSkillProvisionImpulse]:
        return list(map(lambda x: UpdatedSkillProvisionImpulse(x), self.dictionary.get("skillProvisions")))



import logging
import unittest
import uuid
import pytest

from gaia_sdk.graphql import PerceiveDataImpulse
from gaia_sdk.graphql.request.input.PerceiveActionImpulse import PerceiveActionImpulse
from gaia_sdk.graphql.request.type import Perception
from gaia_sdk.tests.mock import mock_gaia_ref
from gaia_sdk.tests.mock import MockResponse

logging.basicConfig(level=logging.DEBUG)
from rx import operators as ops, pipe


class RxException(Exception):
    pass

class TestPerception(unittest.TestCase):

    def test_perceive_data(self):
        gaia_ref = mock_gaia_ref(lambda x: MockResponse({"data": { "perceive": { "perceiveData": {"id": "asdf"}}}}))
        impulse = PerceiveDataImpulse(str(uuid.uuid4()), "", {})

        result = pipe(ops.first())(gaia_ref.perceive_data(impulse)).run()
        assert result.dictionary.get("id") is not None, "PerceiveData.id is in response"

    def test_perceive_action(self):
        gaia_ref = mock_gaia_ref(lambda x: MockResponse({"data": { "perceive": { "perceiveAction": {"id": "asdf"}}}}))
        impulse = PerceiveActionImpulse(False, str(uuid.uuid4()), "", {})

        result = pipe(ops.first())(gaia_ref.perceive_action(impulse)).run()
        assert result.dictionary.get("id") is not None, "PerceiveAction.id is in response"

    def test_perceive(self):
        gaia_ref = mock_gaia_ref(lambda x: MockResponse({"data": { "perceive": { "perceiveData": {"id": "asdf"}, "perceiveAction": {"id": "qwer"}}}}))
        impulse1 = PerceiveActionImpulse(False, str(uuid.uuid4()), "", {})
        impulse2 = PerceiveDataImpulse(str(uuid.uuid4()), "", {})

        def perceive(p: Perception):
            p.perceive_action(impulse1, lambda x: x.id())
            p.perceive_data(impulse2, lambda x: x.id())

        result = pipe(ops.first())(gaia_ref.perceive(perceive)).run()
        perceiveData = result.dictionary.get("perceiveData")
        perceiveAction = result.dictionary.get("perceiveAction")
        assert perceiveData.get("id") is not None, "PerceiveData.id is in response"
        assert perceiveAction.get("id") is not None, "perceiveAction.id is in response"


if __name__ == '__main__':
    unittest.main()





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Edge(list):
    """
    Represents graph information
    """

    def source(self):
        self.append(lambda x: "source")

    def target(self):
        self.append(lambda x: "target")

    def edge_id(self):
        self.append(lambda x: "edgeId")

    def type(self):
        self.append(lambda x: "type")

    def weight(self):
        self.append(lambda x: "weight")

    def properties(self):
        self.append(lambda x: "properties")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



from typing import ClassVar


class Payload:
    JSON: ClassVar[str] = "json"
    FORM_DATA: ClassVar[str] = "form_data"
    STREAM: ClassVar[str] = "stream"

    data: dict
    payload_type: str
    binaryData: bytes

    def __init__(self, data, payload_type: str, binaryData: bytes):
        self.data = data
        self.payload_type = payload_type
        self.binaryData = binaryData

    def __eq__(self, other):
        return self.data == other.data and self.payload_type == other.payload_type

    def __repr__(self):
        return {'data': self.data, 'payload_type': self.payload_type}

    @staticmethod
    def json(data: dict):
        return Payload(data, Payload.JSON, None)

    @staticmethod
    def form_data(data: dict):
        return Payload(data, Payload.FORM_DATA, None)

    @staticmethod
    def stream(data: bytes):
        return Payload(None, Payload.STREAM, data)



import unittest
import base64

from api.crypto import HMAC


class TestHMAC(unittest.TestCase):

    def test_hash(self):
        hmac = HMAC("secret")
        self.assertEqual(hmac.hash("data"), 'GywWt1vSqHDBFBU8zaW8/KYzFLxyL6Fg1pDeEzzLuds=')
        self.assertEqual(hmac.hash("data"), 'GywWt1vSqHDBFBU8zaW8/KYzFLxyL6Fg1pDeEzzLuds=')

    def test_hash512(self):
        hmac = HMAC("secret")
        toHash= "aGk=_application/json_http_1592924470_353823db-c12b-44b2-b0dc-c4d813c74b24"
        self.assertEqual(hmac.hash512(toHash), 'MzE5ZjQyNzg3ZTgyZGJhNmE3YTBiNjI5ODA5MjIzMzk2YzRhMTg1MmNlOWUwYzhiYTNiZmQ0MTkxY2NlMDg1YTVlMWM0Y2UwM2QzNzNlM2NhYWIxMzcxMTU5MTQxNTJkNzFhMmEwMmY3OGIwNTZmNjA0NTJkZDJlYzg2ZDE1MjU=')

if __name__ == '__main__':
    unittest.main()




from gaia_sdk.graphql.response.type.DeletedApiKeyImpulse import DeletedApiKeyImpulse
from gaia_sdk.graphql.response.type.DeletedFulfilmentImpulse import DeletedFulfilmentImpulse
from gaia_sdk.graphql.response.type.DeletedBehaviourImpulse import DeletedBehaviourImpulse
from gaia_sdk.graphql.response.type.DeletedTenantImpulse import DeletedTenantImpulse
from gaia_sdk.graphql.response.type.DeletedSkillProvisionImpulse import DeletedSkillProvisionImpulse
from gaia_sdk.graphql.response.type.DeletedStatementImpulse import DeletedStatementImpulse
from gaia_sdk.graphql.response.type.DeletedSkillImpulse import DeletedSkillImpulse
from gaia_sdk.graphql.response.type.DeletedRoleImpulse import DeletedRoleImpulse
from gaia_sdk.graphql.response.type.DeletedCodeImpulse import DeletedCodeImpulse
from gaia_sdk.graphql.response.type.DeletedEdgeImpulse import DeletedEdgeImpulse
from gaia_sdk.graphql.response.type.DeletedIdentityImpulse import DeletedIdentityImpulse
from gaia_sdk.graphql.response.type.DeletedIntentImpulse import DeletedIntentImpulse
from gaia_sdk.graphql.response.type.DeletedPromptImpulse import DeletedPromptImpulse
from gaia_sdk.graphql.response.type.DeletedUserImpulse import DeletedUserImpulse
from gaia_sdk.graphql.request.input.DeleteFulfilmentImpulse import DeleteFulfilmentImpulse
from gaia_sdk.graphql.request.input.DeleteCodeImpulse import DeleteCodeImpulse
from gaia_sdk.graphql.request.input.DeleteEdgeImpulse import DeleteEdgeImpulse
from gaia_sdk.graphql.request.input.DeleteBehaviourImpulse import DeleteBehaviourImpulse
from gaia_sdk.graphql.request.input.DeleteSkillImpulse import DeleteSkillImpulse
from gaia_sdk.graphql.request.input.DeleteIdentityImpulse import DeleteIdentityImpulse
from gaia_sdk.graphql.request.input.DeleteUserImpulse import DeleteUserImpulse
from gaia_sdk.graphql.request.input.DeleteApiKeyImpulse import DeleteApiKeyImpulse
from gaia_sdk.graphql.request.input.DeleteStatementImpulse import DeleteStatementImpulse
from gaia_sdk.graphql.request.input.DeletePromptImpulse import DeletePromptImpulse
from gaia_sdk.graphql.request.input.DeleteTenantImpulse import DeleteTenantImpulse
from gaia_sdk.graphql.request.input.DeleteIntentImpulse import DeleteIntentImpulse
from gaia_sdk.graphql.request.input.DeleteRoleImpulse import DeleteRoleImpulse
from gaia_sdk.graphql.request.input.DeleteSkillProvisionImpulse import DeleteSkillProvisionImpulse

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class DeleteKnowledge:
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    deletes a list of identities with the given specifications
    """
    @property
    def identities(self) -> List[DeletedIdentityImpulse]:
        return list(map(lambda x: DeletedIdentityImpulse(x), self.dictionary.get("identities")))
    """
    deletes a list of tenants with the given specifications
    """
    @property
    def tenants(self) -> List[DeletedTenantImpulse]:
        return list(map(lambda x: DeletedTenantImpulse(x), self.dictionary.get("tenants")))
    """
    deletes a list of users with the given specifications
    """
    @property
    def users(self) -> List[DeletedUserImpulse]:
        return list(map(lambda x: DeletedUserImpulse(x), self.dictionary.get("users")))
    """
    deletes a list of api keys with the given specifications
    """
    @property
    def api_keys(self) -> List[DeletedApiKeyImpulse]:
        return list(map(lambda x: DeletedApiKeyImpulse(x), self.dictionary.get("apiKeys")))
    """
    deletes a list of roles with the given specifications
    """
    @property
    def roles(self) -> List[DeletedRoleImpulse]:
        return list(map(lambda x: DeletedRoleImpulse(x), self.dictionary.get("roles")))
    """
    deletes a list of intents with the given specifications
    """
    @property
    def intents(self) -> List[DeletedIntentImpulse]:
        return list(map(lambda x: DeletedIntentImpulse(x), self.dictionary.get("intents")))
    """
    deletes a list of prompts with the given specifications
    """
    @property
    def prompts(self) -> List[DeletedPromptImpulse]:
        return list(map(lambda x: DeletedPromptImpulse(x), self.dictionary.get("prompts")))
    """
    deletes a list of statements with the given specifications
    """
    @property
    def statements(self) -> List[DeletedStatementImpulse]:
        return list(map(lambda x: DeletedStatementImpulse(x), self.dictionary.get("statements")))
    """
    deletes a list of fulfilments with the given specifications
    """
    @property
    def fulfilments(self) -> List[DeletedFulfilmentImpulse]:
        return list(map(lambda x: DeletedFulfilmentImpulse(x), self.dictionary.get("fulfilments")))
    """
    deletes a list of behaviours with the given specifications
    """
    @property
    def behaviours(self) -> List[DeletedBehaviourImpulse]:
        return list(map(lambda x: DeletedBehaviourImpulse(x), self.dictionary.get("behaviours")))
    """
    deletes a list of codes with the given specifications
    """
    @property
    def codes(self) -> List[DeletedCodeImpulse]:
        return list(map(lambda x: DeletedCodeImpulse(x), self.dictionary.get("codes")))
    """
    deletes a list of edges with the given specifications
    """
    @property
    def edges(self) -> List[DeletedEdgeImpulse]:
        return list(map(lambda x: DeletedEdgeImpulse(x), self.dictionary.get("edges")))
    """
    deletes a list of skills with the given specifications
    """
    @property
    def skills(self) -> List[DeletedSkillImpulse]:
        return list(map(lambda x: DeletedSkillImpulse(x), self.dictionary.get("skills")))
    """
    deletes a list of skill provisions with the given specifications
    """
    @property
    def skill_provisions(self) -> List[DeletedSkillProvisionImpulse]:
        return list(map(lambda x: DeletedSkillProvisionImpulse(x), self.dictionary.get("skillProvisions")))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class DeleteRoleImpulse():
    """
    The specification to delete a role instance
    """
    tenantId: str
    roleId: str

    def __init__(self, tenantId: str, roleId: str):
        self.tenantId = tenantId
        self.roleId = roleId

    def __eq__(self, other):
        if type(other) is type(self):
            return self.tenantId == other.tenantId and self.roleId == other.roleId
        return False

    def __repr__(self):
        return {'tenantId': self.tenantId, 'roleId': self.roleId}





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class ApiKey(list):
    """
    Represents api key information
    """

    """
    The api key id
    """
    def api_key_id(self):
        self.append(lambda x: "apiKeyId")

    """
    The name of the api key
    """
    def name(self):
        self.append(lambda x: "name")

    """
    The description of the api key
    """
    def description(self):
        self.append(lambda x: "description")

    """
    The flag to enable the api key
    """
    def enabled(self):
        self.append(lambda x: "enabled")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



from math import ceil
from rx import of
from rx.core.abc import Scheduler
from rx.core.typing import Observable
import logging
import rx
import rx.operators as ops
import time
import uuid

from gaia_sdk.api.DataRef import DataRef
from gaia_sdk.http.request.IdentitySourceRequestImpulse import IdentitySourceRequestImpulse

from gaia_sdk.http.response.IdentityImported import IdentityImported

from gaia_sdk.http.GaiaStreamClient import GaiaStreamClient
from gaia_sdk.http.request.IdentityImportImpulse import IdentityImportImpulse

CHUNK_SIZE = 1024 * 1024 * 5


def check_identity_id(identity_id):
    if identity_id is None:
        return str(uuid.uuid4())
    return identity_id


class IdentityOp:
    def __init__(self, client: GaiaStreamClient, scheduler: Scheduler):
        self._client = client
        self._logger = logging.getLogger("IdentityRef")
        self._scheduler = scheduler

    def export(self, identity_id: str) -> Observable[bytes]:
        r"""Exports the identity with the current identity ID and returns it as bytes.

        :param identity_id: ID of the identity to be exported.
        :return: :class:`Observable[bytes]` object: Identity as bytes.
        :exception HttpError: Error thrown if the export operation fails.
        """
        self._logger.debug(f"Started export of {identity_id}")

        return rx.from_callable(
            lambda: self._client.post_json(IdentitySourceRequestImpulse(identity_id),
                                           url_postfix="/identity/source"),
            self._scheduler) \
            .pipe(
            ops.map(lambda response: response.content))

    def import_identity(self, tenant_id: str, identity_name: str, content: bytes, override: bool = False,
                        identity_id: str = None) -> Observable[IdentityImported]:
        def complete_import(data_ref: DataRef) -> dict:
            self._logger.debug(f"Started import of identity {identity_name}")
            return self._client.post_json(IdentityImportImpulse(data_ref.uri, tenant_id, check_identity_id(identity_id),
                                                                identity_name, override),
                                          "/identity/import").json()

        uri = f"gaia://{tenant_id}/identities/"

        self._logger.debug(f"Started upload to uri {uri}")
        file_name = f"{identity_name}-{round(time.time() * 1000)}"
        new_file_data_ref = DataRef(uri, self._client, self._scheduler) \
            .add(file_name, content, override)

        return new_file_data_ref.pipe(ops.map(complete_import))



class UUID(object):
    value: str

    def __init__(self, value: str):
        self.value = value

    def __eq__(self, other):
        return self.value == other.value

    def __repr__(self):
        return {'value': self.value}

    @staticmethod
    def random_uuid() -> 'UUID':
        import uuid
        return UUID(str(uuid.uuid4()))

    def to_string(self) -> str:
        return self.value

    def to_json(self):
        return self.value





from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class User:
    """
    Represents User information
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    Id of the user
    """
    @property
    def user_id(self) -> Uuid:
        return Uuid(self.dictionary.get("userId"))
    """
    The username of the user
    """
    @property
    def username(self) -> String:
        return String(self.dictionary.get("username"))
    """
    The email of the user
    """
    @property
    def email(self) -> String:
        return String(self.dictionary.get("email"))
    """
    The first name of the user
    """
    @property
    def first_name(self) -> String:
        return String(self.dictionary.get("firstName"))
    """
    The last name of the user
    """
    @property
    def last_name(self) -> String:
        return String(self.dictionary.get("lastName"))
    """
    Is the User using 2FA
    """
    @property
    def using2_f_a(self) -> Boolean:
        return Boolean(self.dictionary.get("using2FA"))
    """
    The tenants of the user
    """
    @property
    def tenants(self) -> List[String]:
        return list(map(lambda x: String(x), self.dictionary.get("tenants")))
    """
    The roles of the user
    """
    @property
    def roles(self) -> List[String]:
        return list(map(lambda x: String(x), self.dictionary.get("roles")))
    """
    The groups of the user
    """
    @property
    def groups(self) -> List[String]:
        return list(map(lambda x: String(x), self.dictionary.get("groups")))
    """
    The permissions of the user
    """
    @property
    def permissions(self) -> List[String]:
        return list(map(lambda x: String(x), self.dictionary.get("permissions")))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class UpdateUserImpulse():
    """
    The specification to update a user instance
    """
    userId: str
    username: str
    email: str
    firstName: str
    lastName: str
    password: str
    using2FA: bool
    tenants: List[str]
    roles: List[str]
    groups: List[str]
    permissions: List[str]

    def __init__(self, userId: str, username: str, email: str, firstName: str, lastName: str, password: str, using2FA: bool, tenants: List[str], roles: List[str], groups: List[str], permissions: List[str]):
        self.userId = userId
        self.username = username
        self.email = email
        self.firstName = firstName
        self.lastName = lastName
        self.password = password
        self.using2FA = using2FA
        self.tenants = tenants
        self.roles = roles
        self.groups = groups
        self.permissions = permissions

    def __eq__(self, other):
        if type(other) is type(self):
            return self.userId == other.userId and self.username == other.username and self.email == other.email and self.firstName == other.firstName and self.lastName == other.lastName and self.password == other.password and self.using2FA == other.using2FA and self.tenants == other.tenants and self.roles == other.roles and self.groups == other.groups and self.permissions == other.permissions
        return False

    def __repr__(self):
        return {'userId': self.userId, 'username': self.username, 'email': self.email, 'firstName': self.firstName, 'lastName': self.lastName, 'password': self.password, 'using2FA': self.using2FA, 'tenants': self.tenants, 'roles': self.roles, 'groups': self.groups, 'permissions': self.permissions}





from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class SkillProvision:
    """
    Represents SkillProvision information
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    Id of the tenant
    """
    @property
    def tenant_id(self) -> Uuid:
        return Uuid(self.dictionary.get("tenantId"))
    """
    Skill reference
    """
    @property
    def reference(self) -> Uuid:
        return Uuid(self.dictionary.get("reference"))
    """
    The name of the skill provision
    """
    @property
    def qualifier(self) -> String:
        return String(self.dictionary.get("qualifier"))
    """
    Detailed description about the skill provision
    """
    @property
    def appendent(self) -> String:
        return String(self.dictionary.get("appendent"))
    """
    The list of labels of the skill provision
    """
    @property
    def label_list(self) -> List[String]:
        return list(map(lambda x: String(x), self.dictionary.get("labelList")))
    """
    The version of the skill
    """
    @property
    def version(self) -> String:
        return String(self.dictionary.get("version"))
    """
    The reference to skill
    """
    @property
    def skill_ref(self) -> String:
        return String(self.dictionary.get("skillRef"))
    """
    CPU
    """
    @property
    def cpu(self) -> Int:
        return Int(self.dictionary.get("cpu"))
    """
    Memory
    """
    @property
    def memory(self) -> Int:
        return Int(self.dictionary.get("memory"))
    """
    Amount of replicas
    """
    @property
    def replicas(self) -> Int:
        return Int(self.dictionary.get("replicas"))
    """
    Whether the skill provision should be enabled or not
    """
    @property
    def enabled(self) -> Boolean:
        return Boolean(self.dictionary.get("enabled"))
    """
    Amount of seconds that the skill requires to be ready
    """
    @property
    def bootstrap_timeout(self) -> Int:
        return Int(self.dictionary.get("bootstrapTimeout"))
    """
    Value-Key pairs with information needed for the skill provision
    """
    @property
    def environment(self) -> Struct:
        return Struct(self.dictionary.get("environment"))
    """
    Whether the skill provision has been built
    """
    @property
    def built(self) -> Boolean:
        return Boolean(self.dictionary.get("built"))
    """
    The current status of the skill provision
    """
    @property
    def status(self) -> Struct:
        return Struct(self.dictionary.get("status"))



class InitBinaryWriteImpulse:
    uri: str
    override: bool

    def __init__(self, uri: str, override: bool):
        self.uri = uri
        self.override = override

    def __eq__(self, other):
        return self.uri == other.uri \
               and self.override == other.override

    def __repr__(self):
        return {'uri': self.uri,
                'override': self.override}



from typing import List


class CompleteBinaryWriteImpulse:
    uri: str
    uploadId: str
    chunks: List[str]

    def __init__(self, uri: str, uploadId: str, chunks: List[str]):
        self.uri = uri
        self.uploadId = uploadId
        self.chunks = chunks

    def __eq__(self, other):
        return self.uri == other.uri and self.uploadId == other.uploadId and self.chunks == other.chunks

    def __repr__(self):
        return {'uri': self.uri, 'uploadId': self.uploadId, 'chunks': self.chunks}




from gaia_sdk.graphql.request.type.Interaction import Interaction
from gaia_sdk.graphql.request.type.Introspection import Introspection
from gaia_sdk.graphql.request.type.Notification import Notification

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Subscription(list):
    """
    the top level subscription type
    """

    def interact(self, config: Callable[['Interaction'], None]):
        def callback(registry: VariableRegistry):
            entity = Interaction()
            config(entity)
            return "interact {" + entity.render(registry) + "}"
        self.append(callback)

    def notify(self, config: Callable[['Notification'], None]):
        def callback(registry: VariableRegistry):
            entity = Notification()
            config(entity)
            return "notify {" + entity.render(registry) + "}"
        self.append(callback)

    """
    Container element for all introspect sensor fields
    """
    def introspect(self, config: Callable[['Introspection'], None]):
        def callback(registry: VariableRegistry):
            entity = Introspection()
            config(entity)
            return "introspect {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.request.type.Experience import Experience
from gaia_sdk.graphql.request.type.Knowledge import Knowledge

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Retrieval(list):

    """
    Container element which collects all information static data
    """
    def knowledge(self, config: Callable[['Knowledge'], None]):
        def callback(registry: VariableRegistry):
            entity = Knowledge()
            config(entity)
            return "knowledge {" + entity.render(registry) + "}"
        self.append(callback)

    """
    Container element which collects all information about runtime data
    """
    def experience(self, config: Callable[['Experience'], None]):
        def callback(registry: VariableRegistry):
            entity = Experience()
            config(entity)
            return "experience {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class ConnectRemoveNodeImpulse():
    """
    The specification to remove an edge of a certain type with a certain target from a node
    """
    type: EdgeType
    target: str

    def __init__(self, type: EdgeType, target: str):
        self.type = type
        self.target = target

    def __eq__(self, other):
        if type(other) is type(self):
            return self.type == other.type and self.target == other.target
        return False

    def __repr__(self):
        return {'type': self.type, 'target': self.target}





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class PerceiveSuggestionImpulse():
    """
    Input for suggestion perception impulse
    """
    data: dict

    def __init__(self, data: dict):
        self.data = data

    def __eq__(self, other):
        if type(other) is type(self):
            return self.data == other.data
        return False

    def __repr__(self):
        return {'data': self.data}



from typing import Callable
from gaia_sdk.graphql.request.type.Query import Query
from gaia_sdk.graphql.request.type.Mutation import Mutation
from gaia_sdk.graphql.request.type.Subscription import Subscription


class GaiaRequest:

    @staticmethod
    def query(config: Callable[[Query], None]):
        request = Query()
        config(request)
        return request

    @staticmethod
    def mutation(config: Callable[[Mutation], None]):
        request = Mutation()
        config(request)
        return request

    @staticmethod
    def subscription(config: Callable[[Subscription], None]):
        request = Subscription()
        config(request)
        return request



import time
from abc import ABC, abstractmethod

import json
from gaia_sdk.api.client_options import ClientOptions
from gaia_sdk.graphql.GaiaScalars import UUID
from gaia_sdk.http.HMACTokenBuilder import HMACTokenBuilder


class GaiaCredentials(ABC):
    @abstractmethod
    def create_auth_header(self, options: ClientOptions, payload: str) -> str:
        pass


class UsernamePasswordCredentials(GaiaCredentials):
    username: str
    password: str

    def __init__(self, username: str, password: str):
        self.username = username
        self.password = password

    def __eq__(self, other):
        return self.username == other.username and self.password == other.password

    def __repr__(self):
        return {'username': self.username, 'password': self.password}

    def create_auth_header(self, options, payload) -> str:
        raise NotImplementedError("Creating an authentication string is not implemented for the used credentials.")


class HMACCredentials(GaiaCredentials):
    apiKey: str
    apiSecret: str

    def __init__(self, apiKey: str, apiSecret: str):
        self.apiKey = apiKey
        self.apiSecret = apiSecret

    def __eq__(self, other):
        return self.apiKey == other.apiKey and self.apiSecret == other.apiSecret

    def __repr__(self):
        return {'apiKey': self.apiKey, 'apiSecret': self.apiSecret}

    def create_auth_header(self, options, payload) -> str:
        timestamp = int(round(time.time()))
        nonce = UUID.random_uuid().value
        return HMACTokenBuilder().with_timestamp(timestamp) \
            .with_client_options(options) \
            .with_nonce(nonce) \
            .with_payload(payload).build()


class JWTCredentials(GaiaCredentials):
    token: str

    def __init__(self, token):
        self.token = token

    def __eq__(self, other):
        return self.token == other.token

    def __repr__(self):
        return {'token': self.token}

    def create_auth_header(self, options, payload) -> str:
        return "Bearer " + options.credentials.token




from gaia_sdk.graphql.response.type.BehaviourExecution import BehaviourExecution
from gaia_sdk.graphql.response.type.SkillProvisionBuildJob import SkillProvisionBuildJob
from gaia_sdk.graphql.response.type.IdentityMetrics import IdentityMetrics
from gaia_sdk.graphql.response.type.BehaviourNodeExecution import BehaviourNodeExecution
from gaia_sdk.graphql.response.type.BehaviourExecutionDetail import BehaviourExecutionDetail

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class Experience:
    """
    Container type for runtime information
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    @property
    def behaviour_execution(self) -> BehaviourExecutionDetail:
        return BehaviourExecutionDetail(self.dictionary.get("behaviourExecution"))
    @property
    def behaviour_executions(self) -> List[BehaviourExecution]:
        return list(map(lambda x: BehaviourExecution(x), self.dictionary.get("behaviourExecutions")))
    @property
    def behaviour_node_executions(self) -> List[BehaviourNodeExecution]:
        return list(map(lambda x: BehaviourNodeExecution(x), self.dictionary.get("behaviourNodeExecutions")))
    @property
    def identity_metrics(self) -> IdentityMetrics:
        return IdentityMetrics(self.dictionary.get("identityMetrics"))
    @property
    def skill_provision_build_jobs(self) -> List[SkillProvisionBuildJob]:
        return list(map(lambda x: SkillProvisionBuildJob(x), self.dictionary.get("skillProvisionBuildJobs")))




from gaia_sdk.graphql.request.type.OnUpdated import OnUpdated
from gaia_sdk.graphql.request.type.OnDeleted import OnDeleted
from gaia_sdk.graphql.request.type.OnCreated import OnCreated

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Notification(list):

    def on_created(self, config: Callable[['OnCreated'], None]):
        def callback(registry: VariableRegistry):
            entity = OnCreated()
            config(entity)
            return "on_created {" + entity.render(registry) + "}"
        self.append(callback)

    def on_updated(self, config: Callable[['OnUpdated'], None]):
        def callback(registry: VariableRegistry):
            entity = OnUpdated()
            config(entity)
            return "on_updated {" + entity.render(registry) + "}"
        self.append(callback)

    def on_deleted(self, config: Callable[['OnDeleted'], None]):
        def callback(registry: VariableRegistry):
            entity = OnDeleted()
            config(entity)
            return "on_deleted {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.response.type.DeleteKnowledge import DeleteKnowledge
from gaia_sdk.graphql.response.type.UpdateKnowledge import UpdateKnowledge
from gaia_sdk.graphql.response.type.CreateKnowledge import CreateKnowledge
from gaia_sdk.graphql.response.type.ConnectKnowledge import ConnectKnowledge

from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class Preservation:
    """
    This type contains all preservation sensor impulses which are used to support
    read/write/delete memory functions in gaia.
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    @property
    def create(self) -> CreateKnowledge:
        return CreateKnowledge(self.dictionary.get("create"))
    @property
    def update(self) -> UpdateKnowledge:
        return UpdateKnowledge(self.dictionary.get("update"))
    @property
    def delete(self) -> DeleteKnowledge:
        return DeleteKnowledge(self.dictionary.get("delete"))
    @property
    def connect(self) -> ConnectKnowledge:
        return ConnectKnowledge(self.dictionary.get("connect"))



import logging
import unittest
from uuid import uuid4

from rx import operators as ops, pipe

from gaia_sdk.graphql.request.type import SkillProvisionBuildJob
from gaia_sdk.tests.mock import mock_gaia_ref
from gaia_sdk.tests.mock import MockResponse
import pytest

logging.basicConfig(level=logging.DEBUG)


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestRetrieval(unittest.TestCase):

    def test_retrieve_codes(self):
        gaia_ref = mock_gaia_ref(
            lambda request: MockResponse({"data": {"retrieve": {"knowledge": {"codes": [{"identityId": "asdf"}]}}}}))

        def config(x):
            x.identity_id()

        result = pipe(ops.first())(gaia_ref.retrieve_codes(str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"

    def test_retrieve_paginated_codes(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "codes": [{"identityId": "i1", "qualifier": "101"}, {"identityId": "i2", "qualifier": "102"},
                      {"identityId": "i3", "qualifier": "103"}, {"identityId": "i4", "qualifier": "104"},
                      {"identityId": "i5", "qualifier": "105"}, {"identityId": "i6", "qualifier": "106"},
                      {"identityId": "i7", "qualifier": "107"}, {"identityId": "i8", "qualifier": "108"},
                      {"identityId": "i9", "qualifier": "109"}, {"identityId": "i10", "qualifier": "110"}]}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()

        result = pipe(ops.to_list())(gaia_ref.retrieve_codes(str(uuid4()), config, 10, 100)).run()

        assert len(result) == 10, "Count does not match"

        latestExpectedIndex = 100
        for i in range(10):
            latestExpectedIndex += 1
            assert result[i].dictionary.get("qualifier") == str(latestExpectedIndex), "index does not match"

    def test_retrieve_code(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"code": {"identityId": "asdf", "reference": "ref1"}}}}}))

        def config(x):
            x.identity_id()
            x.reference()

        result = pipe(ops.first())(gaia_ref.retrieve_code(str(uuid4()), str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"
        assert result.dictionary.get("reference") is not None, "Reference is in response"

    def test_retrieve_identities(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "identities": [{"identityId": "asdf", "qualifier": "q1", "availableLanguages": {"de": "Deutsch"}}]}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()
            x.available_languages()

        result = pipe(ops.first())(gaia_ref.retrieve_identities(config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"
        assert result.dictionary.get("availableLanguages") is not None, "Available Languages are in response"

    def test_retrieve_paginated_identities(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "identities": [{"identityId": "i1", "qualifier": "101", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i2", "qualifier": "102", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i3", "qualifier": "103", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i4", "qualifier": "104", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i5", "qualifier": "105", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i6", "qualifier": "106", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i7", "qualifier": "107", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i8", "qualifier": "108", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i9", "qualifier": "109", "availableLanguages": {"de": "Deutsch"}},
                           {"identityId": "i10", "qualifier": "110", "availableLanguages": {"de": "Deutsch"}}]}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()
            x.available_languages()

        result = pipe(ops.to_list())(gaia_ref.retrieve_identities(config, 10, 100)).run()

        assert len(result) == 10, "Count does not match"

        latestExpectedIndex = 100
        for i in range(10):
            latestExpectedIndex += 1
            assert result[i].dictionary.get("qualifier") == str(latestExpectedIndex), "index does not match"
            assert result[i].dictionary.get("availableLanguages") is not None, "index does not match"

    def test_retrieve_identity(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "identity": {"identityId": "asdf", "qualifier": "q1", "availableLanguages": {"de": "Deutsch"}}}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()
            x.available_languages()

        result = pipe(ops.first())(gaia_ref.retrieve_identity(str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"
        assert result.dictionary.get("qualifier") is not None, "Qualifier is in response"
        assert result.dictionary.get("availableLanguages") is not None, "Available Languages are in response"

    def test_retrieve_intents(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"intents": [{"identityId": "asdf", "qualifier": "q1"}]}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()

        result = pipe(ops.first())(gaia_ref.retrieve_intents(str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"

    def test_retrieve_paginated_intents(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "intents": [{"identityId": "i1", "qualifier": "101"}, {"identityId": "i2", "qualifier": "102"},
                        {"identityId": "i3", "qualifier": "103"}, {"identityId": "i4", "qualifier": "104"},
                        {"identityId": "i5", "qualifier": "105"}, {"identityId": "i6", "qualifier": "106"},
                        {"identityId": "i7", "qualifier": "107"}, {"identityId": "i8", "qualifier": "108"},
                        {"identityId": "i9", "qualifier": "109"}, {"identityId": "i10", "qualifier": "110"}]}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()

        result = pipe(ops.to_list())(gaia_ref.retrieve_intents(str(uuid4()), config, 10, 100)).run()

        assert len(result) == 10, "Count does not match"

        latestExpectedIndex = 100
        for i in range(10):
            latestExpectedIndex += 1
            assert result[i].dictionary.get("qualifier") == str(latestExpectedIndex), "index does not match"

    def test_retrieve_intent(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {
            "retrieve": {"knowledge": {"intent": {"identityId": "asdf", "reference": "ref1", "qualifier": "q1"}}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()
            x.reference()

        result = pipe(ops.first())(gaia_ref.retrieve_intent(str(uuid4()), str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"
        assert result.dictionary.get("reference") is not None, "Reference is in response"

    def test_retrieve_prompts(self):
        gaia_ref = mock_gaia_ref(
            lambda request: MockResponse({"data": {"retrieve": {"knowledge": {"prompts": [{"identityId": "asdf"}]}}}}))

        def config(x):
            x.identity_id()

        result = pipe(ops.first())(gaia_ref.retrieve_prompts(str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"

    def test_retrieve_paginated_prompts(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "prompts": [{"identityId": "i1", "qualifier": "101"}, {"identityId": "i2", "qualifier": "102"},
                        {"identityId": "i3", "qualifier": "103"}, {"identityId": "i4", "qualifier": "104"},
                        {"identityId": "i5", "qualifier": "105"}, {"identityId": "i6", "qualifier": "106"},
                        {"identityId": "i7", "qualifier": "107"}, {"identityId": "i8", "qualifier": "108"},
                        {"identityId": "i9", "qualifier": "109"}, {"identityId": "i10", "qualifier": "110"}]}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()

        result = pipe(ops.to_list())(gaia_ref.retrieve_prompts(str(uuid4()), config, 10, 100)).run()

        assert len(result) == 10, "Count does not match"

        latestExpectedIndex = 100
        for i in range(10):
            latestExpectedIndex += 1
            assert result[i].dictionary.get("qualifier") == str(latestExpectedIndex), "index does not match"

    def test_retrieve_prompt(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"prompt": {"identityId": "asdf", "reference": "ref1"}}}}}))

        def config(x):
            x.identity_id()
            x.reference()

        result = pipe(ops.first())(gaia_ref.retrieve_prompt(str(uuid4()), str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"
        assert result.dictionary.get("reference") is not None, "Reference is in response"

    def test_retrieve_fulfilments(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"fulfilments": [{"identityId": "asdf"}]}}}}))

        def config(x):
            x.identity_id()

        result = pipe(ops.first())(gaia_ref.retrieve_fulfilments(str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"

    def test_retrieve_paginated_fulfilments(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "fulfilments": [{"identityId": "i1", "qualifier": "101"}, {"identityId": "i2", "qualifier": "102"},
                            {"identityId": "i3", "qualifier": "103"}, {"identityId": "i4", "qualifier": "104"},
                            {"identityId": "i5", "qualifier": "105"}, {"identityId": "i6", "qualifier": "106"},
                            {"identityId": "i7", "qualifier": "107"}, {"identityId": "i8", "qualifier": "108"},
                            {"identityId": "i9", "qualifier": "109"}, {"identityId": "i10", "qualifier": "110"}]}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()

        result = pipe(ops.to_list())(gaia_ref.retrieve_fulfilments(str(uuid4()), config, 10, 100)).run()

        assert len(result) == 10, "Count does not match"

        latestExpectedIndex = 100
        for i in range(10):
            latestExpectedIndex += 1
            assert result[i].dictionary.get("qualifier") == str(latestExpectedIndex), "index does not match"

    def test_retrieve_fulfilment(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"fulfilment": {"identityId": "asdf", "reference": "ref1"}}}}}))

        def config(x):
            x.identity_id()
            x.reference()

        result = pipe(ops.first())(gaia_ref.retrieve_fulfilment(str(uuid4()), str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"
        assert result.dictionary.get("reference") is not None, "Reference is in response"

    def test_retrieve_statements(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"statements": [{"identityId": "asdf"}]}}}}))

        def config(x):
            x.identity_id()

        result = pipe(ops.first())(gaia_ref.retrieve_statements(str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"

    def test_retrieve_paginated_statements(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "statements": [{"identityId": "i1", "qualifier": "101"}, {"identityId": "i2", "qualifier": "102"},
                           {"identityId": "i3", "qualifier": "103"}, {"identityId": "i4", "qualifier": "104"},
                           {"identityId": "i5", "qualifier": "105"}, {"identityId": "i6", "qualifier": "106"},
                           {"identityId": "i7", "qualifier": "107"}, {"identityId": "i8", "qualifier": "108"},
                           {"identityId": "i9", "qualifier": "109"}, {"identityId": "i10", "qualifier": "110"}]}}}}))

        def config(x):
            x.identity_id()
            x.qualifier()

        result = pipe(ops.to_list())(gaia_ref.retrieve_statements(str(uuid4()), config, 10, 100)).run()

        assert len(result) == 10, "Count does not match"

        latestExpectedIndex = 100
        for i in range(10):
            latestExpectedIndex += 1
            assert result[i].dictionary.get("qualifier") == str(latestExpectedIndex), "index does not match"

    def test_retrieve_statement(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"statement": {"identityId": "asdf", "reference": "ref1"}}}}}))

        def config(x):
            x.identity_id()
            x.reference()

        result = pipe(ops.first())(gaia_ref.retrieve_statement(str(uuid4()), str(uuid4()), config)).run()
        assert result.dictionary.get("identityId") is not None, "IdentityId is in response"
        assert result.dictionary.get("reference") is not None, "Reference is in response"

    def test_retrieve_edges(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"edges": [{"source": "asdf", "target": "q1"}]}}}}))

        def config(x):
            x.source()
            x.target()

        result = pipe(ops.first())(gaia_ref.retrieve_edges(str(uuid4()), config)).run()
        assert result.dictionary.get("source") is not None, "source is in response"
        assert result.dictionary.get("target") is not None, "target is in response"

    def test_retrieve_paginated_edges(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {"retrieve": {"knowledge": {
            "edges": [{"source": "i1", "target": "101", "type": "101"},
                      {"source": "i2", "target": "102", "type": "102"},
                      {"source": "i3", "target": "103", "type": "103"},
                      {"source": "i4", "target": "104", "type": "104"},
                      {"source": "i5", "target": "105", "type": "105"},
                      {"source": "i6", "target": "106", "type": "106"},
                      {"source": "i7", "target": "107", "type": "107"},
                      {"source": "i8", "target": "108", "type": "108"},
                      {"source": "i9", "target": "109", "type": "109"},
                      {"source": "i10", "target": "110", "type": "110"}]}}}}))

        def config(x):
            x.source()
            x.target()
            x.type()

        result = pipe(ops.to_list())(gaia_ref.retrieve_edges(str(uuid4()), config, 10, 100)).run()

        assert len(result) == 10, "Count does not match"

        latestExpectedIndex = 100
        for i in range(10):
            latestExpectedIndex += 1
            assert result[i].dictionary.get("type") == str(latestExpectedIndex), "index does not match"

    def test_retrieve_edge(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"knowledge": {"edge": {"source": "asdf", "target": "ref1"}}}}}))

        def config(x):
            x.source()
            x.target()

        result = pipe(ops.first())(gaia_ref.retrieve_edge(str(uuid4()), str(uuid4()), config)).run()
        assert result.dictionary.get("source") is not None, "source is in response"
        assert result.dictionary.get("target") is not None, "target is in response"

    def test_retrieve_behaviour_execution(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse({"data": {
            "retrieve": {"experience": {"behaviourExecution": {"processInstanceId": 'i1', "behaviourId": '101'}}}}}))

        def config(x):
            x.identity_id()
            x.process_instance_id()

        result = pipe(ops.first())(gaia_ref.retrieve_behaviour_execution(str(uuid4()), str(uuid4()), config)).run()
        assert result.dictionary.get("processInstanceId") is not None, "processInstanceId missing"
        assert result.dictionary.get("behaviourId") is not None, "behaviourId missing"

    def test_retrieve_paginated_behaviour_executions(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {
                "data": {
                    "retrieve": {
                        "experience": {
                            "behaviourExecutions": [{
                                "processInstanceId": 'i1',
                                "behaviourId": '101',
                                "state": '101'
                            }, {"processInstanceId": 'i2', "behaviourId": '102', "state": '102'}]
                        }
                    }
                }
            }
        ))

        def config(x):
            x.process_instance_id()
            x.behaviour_id()
            x.state()

        result = pipe(ops.to_list())(
            gaia_ref.retrieve_behaviour_executions(str(uuid4()), config, 100, 10, None, None)).run()

        assert (len(result)) == 2, "unexpected executions count"
        for r in result:
            assert r.dictionary.get("processInstanceId") is not None, "processInstanceId missing"
            assert r.dictionary.get("behaviourId") is not None, "behaviourId missing"

    def test_retrieve_build_jobs(self):
        gaia_ref = mock_gaia_ref(lambda request: MockResponse(
            {"data": {"retrieve": {"experience": {"skillProvisionBuildJobs": [
                {"skillRef": "44d44584-90b4-4770-b45a-d535392b5031",
                 "provisionRef": "bf8929ab-b432-470e-acb2-bce380f61333", "name": "first-7ac19-build",
                 "status": {"running": "0", "failures": [
                     {"reason": "Failed with an Error", "failureType": "FAILED_UNKNOWN", "exitCode": "1",
                      "logs": "everything is kaputt!",
                      "affectedContainer": "first-dl-init"}], "health": "UNHEALTHY"}},
                {"skillRef": "28c79644-3512-4751-9a0e-6a9b2ef0fdd2",
                 "provisionRef": "a11c745c-72b3-4a75-9f94-ece2a95495d6", "name": "skill-echo-demo-1-fc3bf-build",
                 "status": {"running": "0", "failures": [], "health": "COMPLETED"}}]}}}}
        ))

        def config(x: SkillProvisionBuildJob):
            x.skill_ref()

        result = pipe(ops.to_list())(gaia_ref.retrieve_skill_provision_build_jobs(str(uuid4()), config)).run()

        assert len(result) == 2
        assert result[0].skill_ref == "44d44584-90b4-4770-b45a-d535392b5031"
        assert result[1].provision_ref == "a11c745c-72b3-4a75-9f94-ece2a95495d6" #TODO: something is wrong here...


if __name__ == '__main__':
    unittest.main()




from gaia_sdk.graphql.request.type.PerceivedImpulse import PerceivedImpulse
from gaia_sdk.graphql.request.input.PerceiveReceptionImpulse import PerceiveReceptionImpulse
from gaia_sdk.graphql.request.input.PerceiveSuggestionImpulse import PerceiveSuggestionImpulse
from gaia_sdk.graphql.request.input.PerceiveButtonImpulse import PerceiveButtonImpulse
from gaia_sdk.graphql.request.input.PerceiveUtteranceImpulse import PerceiveUtteranceImpulse
from gaia_sdk.graphql.request.input.PerceiveSubmitImpulse import PerceiveSubmitImpulse

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Conversational(list):
    """
    Type which contains all impulses needed for the maintainence of a conversation
    """

    """
    Utterance perception impulse used to send an utterance text to gaia
    """
    def perceive_utterance(self, impulse: PerceiveUtteranceImpulse, config: Callable[['PerceivedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = PerceivedImpulse()
            config(entity)
            return f'perceiveUtterance(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    Button perception impulse used to send a button action to gaia
    """
    def perceive_button(self, impulse: PerceiveButtonImpulse, config: Callable[['PerceivedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = PerceivedImpulse()
            config(entity)
            return f'perceiveButton(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    Submit perception impulse used to send a submit action to gaia
    """
    def perceive_submit(self, impulse: PerceiveSubmitImpulse, config: Callable[['PerceivedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = PerceivedImpulse()
            config(entity)
            return f'perceiveSubmit(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    Reception perception impulse used to send a reception to gaia
    """
    def perceive_reception(self, impulse: PerceiveReceptionImpulse, config: Callable[['PerceivedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = PerceivedImpulse()
            config(entity)
            return f'perceiveReception(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    Suggestion perception impulse used to send a suggestion action to gaia
    """
    def perceive_suggestion(self, impulse: PerceiveSuggestionImpulse, config: Callable[['PerceivedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = PerceivedImpulse()
            config(entity)
            return f'perceiveSuggestion(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class SkillProvisionBuildJob(list):

    """
    Id of the tenant
    """
    def tenant_id(self):
        self.append(lambda x: "tenantId")

    """
    Reference to the skill provision for that build job
    """
    def provision_ref(self):
        self.append(lambda x: "provisionRef")

    """
    Reference to the skill
    """
    def skill_ref(self):
        self.append(lambda x: "skillRef")

    """
    The name of the build job
    """
    def name(self):
        self.append(lambda x: "name")

    """
    The current status of the build job
    """
    def status(self):
        self.append(lambda x: "status")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Skill(list):
    """
    Represents Skill information
    """

    """
    Id of the tenant
    """
    def tenant_id(self):
        self.append(lambda x: "tenantId")

    """
    Skill reference
    """
    def reference(self):
        self.append(lambda x: "reference")

    """
    The name of the skill
    """
    def qualifier(self):
        self.append(lambda x: "qualifier")

    """
    Detailed description about the skill
    """
    def appendent(self):
        self.append(lambda x: "appendent")

    """
    The list of labels of the skill
    """
    def label_list(self):
        self.append(lambda x: "labelList")

    """
    The uri of the repository where the skill is
    """
    def repository_uri(self):
        self.append(lambda x: "repositoryUri")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))




from gaia_sdk.graphql.request.type.Evaluation import Evaluation
from gaia_sdk.graphql.request.type.Preservation import Preservation
from gaia_sdk.graphql.request.type.Practice import Practice
from gaia_sdk.graphql.request.type.Perception import Perception
from gaia_sdk.graphql.request.type.Activation import Activation

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Mutation(list):
    """
    The top level mutation type
    """

    """
    Sensor impulses for all perception based functions.
        Perceptions are used to invoke events within gaia.
    """
    def perceive(self, config: Callable[['Perception'], None]):
        def callback(registry: VariableRegistry):
            entity = Perception()
            config(entity)
            return "perceive {" + entity.render(registry) + "}"
        self.append(callback)

    """
    Sensor impulses for all practice based functions.
        Practices are used to transfer skills to gaia and to train them.
    """
    def practice(self, config: Callable[['Practice'], None]):
        def callback(registry: VariableRegistry):
            entity = Practice()
            config(entity)
            return "practice {" + entity.render(registry) + "}"
        self.append(callback)

    """
    Sensor impulses for all preservation based functions.
        Preservations are used to invoke create/update/delete functions.
    """
    def preserve(self, config: Callable[['Preservation'], None]):
        def callback(registry: VariableRegistry):
            entity = Preservation()
            config(entity)
            return "preserve {" + entity.render(registry) + "}"
        self.append(callback)

    """
    Container element for all evaluate sensor fields.
        Evaluations are used to invoke skills and to return the result.
    """
    def evaluate(self, config: Callable[['Evaluation'], None]):
        def callback(registry: VariableRegistry):
            entity = Evaluation()
            config(entity)
            return "evaluate {" + entity.render(registry) + "}"
        self.append(callback)

    """
    Container element for all evaluate sensor fields.
        The activation can be used to unseal the vault or to grant access to an user.
    """
    def activate(self, config: Callable[['Activation'], None]):
        def callback(registry: VariableRegistry):
            entity = Activation()
            config(entity)
            return "activate {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



import unittest

from gaia_sdk.api.GaiaCredentials import HMACCredentials
from gaia_sdk.api.client_options import ClientOptions
from gaia_sdk.http.HMACTokenBuilder import HMACTokenBuilder


class TestHMACTokenBuilder(unittest.TestCase):

    def test_generate_token(self):
        options = ClientOptions(HMACCredentials("apiKey", "secret"))
        timestamp = int(1592924470)
        payload = "hi"
        nonce = "353823db-c12b-44b2-b0dc-c4d813c74b24"
        expected_token = "HMAC-SHA512 apiKey_ZmQ4ZTE2ZGVlYWIyZjIxODgyN2MxYzY3NWNhZjBkMWYxZjdmZmRkYjRhM2U4MWI5MWVkNzc3M2QzYjkwYmM1MWI2YTU1NzY1Y2EzZmRkMTNiZjgyZDY0NzJiZmFkZGM0NTM1MGY1NzdiYmNjNzM3YzZmMzczZTZmMGM0ZWExMjE=_1592924470_353823db-c12b-44b2-b0dc-c4d813c74b24"
        token = HMACTokenBuilder().with_payload(payload) \
            .with_nonce(nonce) \
            .with_timestamp(timestamp) \
            .with_client_options(options).build()
        self.assertEqual(token, expected_token)


if __name__ == '__main__':
    unittest.main()




from gaia_sdk.graphql.request.type.SyncSkillEvaluation import SyncSkillEvaluation
from gaia_sdk.graphql.request.type.AsyncSkillEvaluation import AsyncSkillEvaluation

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class SkillEvaluation(list):

    def sync_eval(self, impulse: str, config: Callable[['SyncSkillEvaluation'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = SyncSkillEvaluation()
            config(entity)
            return f'syncEval(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def async_eval(self, impulse: str, config: Callable[['AsyncSkillEvaluation'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = AsyncSkillEvaluation()
            config(entity)
            return f'asyncEval(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



from typing import Dict
from typing import Any


class BinaryWriteChunkImpulse:
    uri: str
    upload_id: str
    ordinal: int
    size_in_bytes: int
    file: bytes

    def __init__(self, uri: str, upload_id: str, ordinal: int, size_in_bytes: int, file: bytes):
        self.uri = uri
        self.upload_id = upload_id
        self.ordinal = ordinal
        self.size_in_bytes = size_in_bytes
        self.file = file

    def __eq__(self, other):
        return self.uri == other.uri \
               and self.upload_id == other.upload_id \
               and self.ordinal == other.ordinal \
               and self.size_in_bytes == other.size_in_bytes \
               and self.file == other.file

    def __repr__(self):
        return {'uri': self.uri,
                'upload_id': self.upload_id,
                'ordinal': self.ordinal,
                'size_in_bytes': self.size_in_bytes,
                'file': self.file}

    def request_parameters(self) -> Dict[str, Any]:
        return {
            "uri": self.uri,
            "uploadId": self.upload_id,
            "ordinal": self.ordinal,
            "sizeInBytes": self.size_in_bytes
        }

    def data(self) -> bytes:
        return self.file




from gaia_sdk.graphql.request.type.DeleteKnowledge import DeleteKnowledge
from gaia_sdk.graphql.request.type.UpdateKnowledge import UpdateKnowledge
from gaia_sdk.graphql.request.type.CreateKnowledge import CreateKnowledge
from gaia_sdk.graphql.request.type.ConnectKnowledge import ConnectKnowledge

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Preservation(list):
    """
    This type contains all preservation sensor impulses which are used to support
    read/write/delete memory functions in gaia.
    """

    def create(self, config: Callable[['CreateKnowledge'], None]):
        def callback(registry: VariableRegistry):
            entity = CreateKnowledge()
            config(entity)
            return "create {" + entity.render(registry) + "}"
        self.append(callback)

    def update(self, config: Callable[['UpdateKnowledge'], None]):
        def callback(registry: VariableRegistry):
            entity = UpdateKnowledge()
            config(entity)
            return "update {" + entity.render(registry) + "}"
        self.append(callback)

    def delete(self, config: Callable[['DeleteKnowledge'], None]):
        def callback(registry: VariableRegistry):
            entity = DeleteKnowledge()
            config(entity)
            return "delete {" + entity.render(registry) + "}"
        self.append(callback)

    def connect(self, config: Callable[['ConnectKnowledge'], None]):
        def callback(registry: VariableRegistry):
            entity = ConnectKnowledge()
            config(entity)
            return "connect {" + entity.render(registry) + "}"
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class BehaviourExecution(list):
    """
    Represents behaviour execution information
    """

    def process_instance_id(self):
        self.append(lambda x: "processInstanceId")

    def identity_id(self):
        self.append(lambda x: "identityId")

    def state(self):
        self.append(lambda x: "state")

    def name(self):
        self.append(lambda x: "name")

    def duration(self):
        self.append(lambda x: "duration")

    def behaviour_id(self):
        self.append(lambda x: "behaviourId")

    def created(self):
        self.append(lambda x: "created")

    def updated(self):
        self.append(lambda x: "updated")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



import math
from math import ceil
from rx import of
from rx.core.abc import Scheduler
from rx.core.typing import Observable
from typing import List, Callable
import functools
import logging
import rx
import rx.operators as ops

from gaia_sdk.http.GaiaStreamClient import GaiaStreamClient
from gaia_sdk.http.request.BinaryReadImpulse import BinaryReadImpulse
from gaia_sdk.http.request.BinaryWriteChunkImpulse import BinaryWriteChunkImpulse
from gaia_sdk.http.request.CompleteBinaryWriteImpulse import CompleteBinaryWriteImpulse
from gaia_sdk.http.request.InitBinaryWriteImpulse import InitBinaryWriteImpulse
from gaia_sdk.http.request.ListFilesImpulse import ListFilesImpulse
from gaia_sdk.http.request.RemoveFileImpulse import RemoveFileImpulse
from gaia_sdk.http.response.BinaryChunkWritten import BinaryChunkWritten
from gaia_sdk.http.response.FileListing import FileListing
from gaia_sdk.http.response.FileRemoved import FileRemoved
from gaia_sdk.http.response.InitializedBinaryWrite import InitializedBinaryWrite

CHUNK_SIZE = 1024 * 1024 * 5


class DataRefRequestConfig:
    def __init__(self, on_upload_progress: Callable[[int], None]):
        self.on_upload_progress = on_upload_progress

    def on_upload_progress(self, progress: int):
        """Return current upload progress"""
        pass


class DataRef:
    def __init__(self, uri: str, client: GaiaStreamClient, scheduler: Scheduler):
        self.uri = uri
        self.client = client
        self.scheduler = scheduler
        self.logger = logging.getLogger("HttpTransporter")

    def add(self, file_name: str, content: bytes, override: bool = False,
            config: DataRefRequestConfig = None) -> Observable['DataRef']:
        r"""Uploads a file with a given name and content to the storage.

        :param file_name: Name of the file to be uploaded.
        :param content: Content of the file to be uploaded in bytes.
        :param override: (optional) Flag to specify if existing files should be overwritten. If set to false, the method
        :param config: (optional) Interface which currently only contains onUploadProgress callback
        will throw an exception when trying to overwrite an existing file.
        :return: :class:`Observable[DataRef]` object: Reference to the newly uploaded file
        :exception HttpError: Error thrown if the upload fails due to override being set to false.
        """
        file_uri = DataRef.concat_uri(self.uri, file_name)
        number_of_chunks = ceil(len(content) / CHUNK_SIZE)
        upload = DataUpload(file_uri, content, number_of_chunks, override, config)

        def execute_upload() -> DataRef:
            self.logger.debug(f"Started upload to uri {file_uri}")
            new_file_data_ref = upload.execute(self.client, self.scheduler)
            self.logger.debug(f"Finished upload to uri {file_uri}")
            return new_file_data_ref

        return rx.from_callable(execute_upload)

    def list(self) -> Observable[List[FileListing]]:
        r"""Lists all files sharing the current uri as prefix.

        :return: :class:`Observable[List[FileListing]]` object: List of files that share the current uri as prefix.
        :exception HttpError: Error thrown if the list operation fails.
        """
        def execute_list() -> List[FileListing]:
            self.logger.debug(f"Started list at uri {self.uri}")
            response = self.client.post_json(ListFilesImpulse(self.uri), "/data/list").json()
            self.logger.debug(f"Completed list at uri {self.uri}")
            return [FileListing(listing) for listing in response]

        return rx.from_callable(execute_list)

    def as_bytes(self) -> Observable[bytes]:
        r"""Downloads the file at the current uri and returns it as bytes.

        :return: :class:`Observable[bytes]` object: File as bytes.
        :exception HttpError: Error thrown if the download operation fails.
        """
        return rx.from_callable(
            lambda: self.client.post_json(BinaryReadImpulse(self.uri),
                                          url_postfix="/data/source"),
            self.scheduler) \
            .pipe(
            ops.map(lambda response: response.content))

    def as_stream(self):
        r"""Not implemented in backend"""
        pass

    def remove(self) -> Observable[FileRemoved]:
        r"""Removes the file at the current uri and provides information if it existed.

        :return: :class:`Observable[FileRemoved]` object: Response to remove request that contains information if the
        file existed.
        :exception HttpError: Error thrown if the remove operation fails.
        """
        return self.remove_file("")

    def remove_file(self, file_name: str) -> Observable[FileRemoved]:
        r"""Removes the specified file in the directory at the current uri.

        :param file_name: Name of the file at the current uri to be removed.
        :return: :class:`Observable[FileRemoved]` object: Response to remove request that contains information if the
        file existed.
        :exception HttpError: Error thrown if the remove operation fails.
        """
        def execute_remove_file() -> FileRemoved:
            file_uri = DataRef.concat_uri(self.uri, file_name)
            self.logger.debug(f"Started removing file at {file_uri}")
            response = self.client.post_json(RemoveFileImpulse(file_uri), "/data/remove").json()
            self.logger.debug(f"Completed removing file at {file_uri}")
            return FileRemoved(response)

        return rx.from_callable(execute_remove_file)

    def append(self):
        r"""Not implemented in backend"""
        pass

    @staticmethod
    def concat_uri(*parts: str):
        return functools.reduce(lambda uri, part: f"{uri.rstrip('/')}/{part.lstrip('/')}", parts).rstrip('/')


class DataUpload:
    uri: str
    content: bytes
    number_of_chunks: int
    override: bool = False
    config: DataRefRequestConfig = None

    def __init__(self, uri: str, content: bytes, number_of_chunks: int, override: bool = False,
                 config: DataRefRequestConfig = None):
        self.uri = uri
        self.content = content
        self.number_of_chunks = number_of_chunks
        self.override = override
        self.config = config

    def __eq__(self, other):
        return self.uri == other.uri \
               and self.content == other.content \
               and self.number_of_chunks == other.number_of_chunks \
               and self.override == other.override \
               and self.config == other.config

    def __repr__(self):
        return {'uri': self.uri,
                'content': self.content,
                'number_of_chunks': self.number_of_chunks,
                'override': self.override,
                'config': self.config}

    def execute(self, client: GaiaStreamClient, scheduler: Scheduler) -> DataRef:
        upload_id = self.init_upload(client).upload_id
        chunk_ids = self.upload_chunks(client, upload_id)
        self.complete_upload(client, upload_id, chunk_ids)
        return DataRef(self.uri, client, scheduler)

    def init_upload(self, client: GaiaStreamClient) -> InitializedBinaryWrite:
        response = client.post_json(InitBinaryWriteImpulse(self.uri,
                                                           self.override), "/data/sink/init")
        return InitializedBinaryWrite(response.json())

    def upload_chunks(self, client: GaiaStreamClient, upload_id: str) -> List[str]:
        chunk_ids: List[str] = []
        current_chunk: int = 1
        for index in range(self.number_of_chunks):
            chunk_data = self.content[index * CHUNK_SIZE: min((index + 1) * CHUNK_SIZE, len(self.content))]
            chunk_impulse = BinaryWriteChunkImpulse(self.uri, upload_id, index + 1, len(chunk_data), chunk_data)
            response = client.post_stream(chunk_impulse.data(), chunk_impulse.request_parameters(),
                                          "/data/sink/chunk").json()
            chunk_ids.insert(index, BinaryChunkWritten(response).chunk_id)
            if self.config is not None:
                progress = (100 * current_chunk) / self.number_of_chunks
                current_chunk += 1
                self.config.on_upload_progress(math.ceil(progress))
        return chunk_ids

    def complete_upload(self, client: GaiaStreamClient, upload_id: str, chunk_ids: List[str]) -> dict:
        return client.post_json(CompleteBinaryWriteImpulse(self.uri, upload_id, chunk_ids),
                                "/data/sink/complete").json()




from gaia_sdk.graphql.request.type.CreatedEdgeImpulse import CreatedEdgeImpulse
from gaia_sdk.graphql.request.type.CreatedCodeImpulse import CreatedCodeImpulse
from gaia_sdk.graphql.request.type.CreatedPromptImpulse import CreatedPromptImpulse
from gaia_sdk.graphql.request.type.CreatedIntentImpulse import CreatedIntentImpulse
from gaia_sdk.graphql.request.type.CreatedSkillImpulse import CreatedSkillImpulse
from gaia_sdk.graphql.request.type.CreatedFulfilmentImpulse import CreatedFulfilmentImpulse
from gaia_sdk.graphql.request.type.CreatedUserImpulse import CreatedUserImpulse
from gaia_sdk.graphql.request.type.CreatedSkillProvisionImpulse import CreatedSkillProvisionImpulse
from gaia_sdk.graphql.request.type.CreatedTenantImpulse import CreatedTenantImpulse
from gaia_sdk.graphql.request.type.CreatedStatementImpulse import CreatedStatementImpulse
from gaia_sdk.graphql.request.type.CreatedRoleImpulse import CreatedRoleImpulse
from gaia_sdk.graphql.request.type.CreatedBehaviourImpulse import CreatedBehaviourImpulse
from gaia_sdk.graphql.request.type.CreatedIdentityImpulse import CreatedIdentityImpulse
from gaia_sdk.graphql.request.type.CreatedApiKeyImpulse import CreatedApiKeyImpulse
from gaia_sdk.graphql.request.input.CreateTenantImpulse import CreateTenantImpulse
from gaia_sdk.graphql.request.input.CreateSkillImpulse import CreateSkillImpulse
from gaia_sdk.graphql.request.input.CreateEdgeImpulse import CreateEdgeImpulse
from gaia_sdk.graphql.request.input.CreateIdentityImpulse import CreateIdentityImpulse
from gaia_sdk.graphql.request.input.CreateCodeImpulse import CreateCodeImpulse
from gaia_sdk.graphql.request.input.CreateFulfilmentImpulse import CreateFulfilmentImpulse
from gaia_sdk.graphql.request.input.CreateStatementImpulse import CreateStatementImpulse
from gaia_sdk.graphql.request.input.CreateSkillProvisionImpulse import CreateSkillProvisionImpulse
from gaia_sdk.graphql.request.input.CreateRoleImpulse import CreateRoleImpulse
from gaia_sdk.graphql.request.input.CreateIntentImpulse import CreateIntentImpulse
from gaia_sdk.graphql.request.input.CreatePromptImpulse import CreatePromptImpulse
from gaia_sdk.graphql.request.input.CreateBehaviourImpulse import CreateBehaviourImpulse
from gaia_sdk.graphql.request.input.CreateUserImpulse import CreateUserImpulse
from gaia_sdk.graphql.request.input.CreateApiKeyImpulse import CreateApiKeyImpulse

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class CreateKnowledge(list):

    """
    creates a list of identities with the given specifications
    """
    def identities(self, impulses: List[CreateIdentityImpulse], config: Callable[['CreatedIdentityImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedIdentityImpulse()
            config(entity)
            return f'identities(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of tenants with the given specifications
    """
    def tenants(self, impulses: List[CreateTenantImpulse], config: Callable[['CreatedTenantImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedTenantImpulse()
            config(entity)
            return f'tenants(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of users with the given specifications
    """
    def users(self, impulses: List[CreateUserImpulse], config: Callable[['CreatedUserImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedUserImpulse()
            config(entity)
            return f'users(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of api keys with the given specifications
    """
    def api_keys(self, impulses: List[CreateApiKeyImpulse], config: Callable[['CreatedApiKeyImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedApiKeyImpulse()
            config(entity)
            return f'apiKeys(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of roles with the given specifications
    """
    def roles(self, impulses: List[CreateRoleImpulse], config: Callable[['CreatedRoleImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedRoleImpulse()
            config(entity)
            return f'roles(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of intents with the given specifications
    """
    def intents(self, impulses: List[CreateIntentImpulse], config: Callable[['CreatedIntentImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedIntentImpulse()
            config(entity)
            return f'intents(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of prompts with the given specifications
    """
    def prompts(self, impulses: List[CreatePromptImpulse], config: Callable[['CreatedPromptImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedPromptImpulse()
            config(entity)
            return f'prompts(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of statements with the given specifications
    """
    def statements(self, impulses: List[CreateStatementImpulse], config: Callable[['CreatedStatementImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedStatementImpulse()
            config(entity)
            return f'statements(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of fulfilments with the given specifications
    """
    def fulfilments(self, impulses: List[CreateFulfilmentImpulse], config: Callable[['CreatedFulfilmentImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedFulfilmentImpulse()
            config(entity)
            return f'fulfilments(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of behaviours with the given specifications
    """
    def behaviours(self, impulses: List[CreateBehaviourImpulse], config: Callable[['CreatedBehaviourImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedBehaviourImpulse()
            config(entity)
            return f'behaviours(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of codes with the given specifications
    """
    def codes(self, impulses: List[CreateCodeImpulse], config: Callable[['CreatedCodeImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedCodeImpulse()
            config(entity)
            return f'codes(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of edges with the given specifications
    """
    def edges(self, impulses: List[CreateEdgeImpulse], config: Callable[['CreatedEdgeImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedEdgeImpulse()
            config(entity)
            return f'edges(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of skills with the given specifications
    """
    def skills(self, impulses: List[CreateSkillImpulse], config: Callable[['CreatedSkillImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedSkillImpulse()
            config(entity)
            return f'skills(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    creates a list of skill provisions with the given specifications
    """
    def skill_provisions(self, impulses: List[CreateSkillProvisionImpulse], config: Callable[['CreatedSkillProvisionImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulses", impulses)
            entity = CreatedSkillProvisionImpulse()
            config(entity)
            return f'skillProvisions(impulses:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import List
Uuid = str
String = str
ISO8601 = str
Struct = dict
Float = float
Int = int
Boolean = bool
from gaia_sdk.graphql.request.enumeration.RuntimeState import RuntimeState
from gaia_sdk.graphql.request.enumeration.SkillState import SkillState
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType

class Statement:
    """
    Represents statement information
    """
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        if type(other) is type(self):
            return self.dictionary == other.dictionary
        return False

    def __repr__(self):
        return {'dictionary': self.dictionary}

    """
    The statement id
    """
    @property
    def identity_id(self) -> Uuid:
        return Uuid(self.dictionary.get("identityId"))
    """
    The statement reference id
    """
    @property
    def reference(self) -> Uuid:
        return Uuid(self.dictionary.get("reference"))
    """
    The name of the statement
    """
    @property
    def qualifier(self) -> String:
        return String(self.dictionary.get("qualifier"))
    """
    Detailed description about the statement
    """
    @property
    def appendent(self) -> String:
        return String(self.dictionary.get("appendent"))
    """
    The utterance dictionary. The key is a language key and the value is a list of utterances
    """
    @property
    def utterance(self) -> Struct:
        return Struct(self.dictionary.get("utterance"))
    """
    The list of labels of the statement
    """
    @property
    def label_list(self) -> List[String]:
        return list(map(lambda x: String(x), self.dictionary.get("labelList")))
    """
    The version of the statement
    """
    @property
    def version(self) -> String:
        return String(self.dictionary.get("version"))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class BehaviourState(list):

    def behaviour_id(self):
        self.append(lambda x: "behaviourId")

    def behaviour_name(self):
        self.append(lambda x: "behaviourName")

    def number_of_executions(self):
        self.append(lambda x: "numberOfExecutions")

    def running(self):
        self.append(lambda x: "running")

    def success(self):
        self.append(lambda x: "success")

    def waiting(self):
        self.append(lambda x: "waiting")

    def failed(self):
        self.append(lambda x: "failed")

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))



class BinaryChunkWritten:
    dictionary: dict

    def __init__(self, dictionary: dict):
        self.dictionary = dictionary

    def __eq__(self, other):
        return self.dictionary == other.dictionary

    def __repr__(self):
        return {'dictionary': self.dictionary}

    @property
    def chunk_id(self) -> str:
        return self.dictionary.get("chunkId")



from gaia_sdk.graphql.request.type.Behaviour import Behaviour as BehaviourReq
from gaia_sdk.graphql.request.type.Code import Code as CodeReq
from gaia_sdk.graphql.request.input.CreateIdentityImpulse import CreateIdentityImpulse
from gaia_sdk.graphql.request.input.UpdateIdentityImpulse import UpdateIdentityImpulse
from gaia_sdk.graphql.request.input.DeleteIdentityImpulse import DeleteIdentityImpulse
from gaia_sdk.graphql.request.input.CreateIntentImpulse import CreateIntentImpulse
from gaia_sdk.graphql.request.input.UpdateIntentImpulse import UpdateIntentImpulse
from gaia_sdk.graphql.request.input.DeleteIntentImpulse import DeleteIntentImpulse
from gaia_sdk.graphql.request.input.CreatePromptImpulse import CreatePromptImpulse
from gaia_sdk.graphql.request.input.UpdatePromptImpulse import UpdatePromptImpulse
from gaia_sdk.graphql.request.input.DeletePromptImpulse import DeletePromptImpulse
from gaia_sdk.graphql.request.input.CreateStatementImpulse import CreateStatementImpulse
from gaia_sdk.graphql.request.input.UpdateStatementImpulse import UpdateStatementImpulse
from gaia_sdk.graphql.request.input.DeleteStatementImpulse import DeleteStatementImpulse
from gaia_sdk.graphql.request.input.CreateFulfilmentImpulse import CreateFulfilmentImpulse
from gaia_sdk.graphql.request.input.UpdateFulfilmentImpulse import UpdateFulfilmentImpulse
from gaia_sdk.graphql.request.input.DeleteFulfilmentImpulse import DeleteFulfilmentImpulse
from gaia_sdk.graphql.request.input.CreateBehaviourImpulse import CreateBehaviourImpulse
from gaia_sdk.graphql.request.input.UpdateBehaviourImpulse import UpdateBehaviourImpulse
from gaia_sdk.graphql.request.input.DeleteBehaviourImpulse import DeleteBehaviourImpulse
from gaia_sdk.graphql.request.input.CreateCodeImpulse import CreateCodeImpulse
from gaia_sdk.graphql.request.input.UpdateCodeImpulse import UpdateCodeImpulse
from gaia_sdk.graphql.request.input.DeleteCodeImpulse import DeleteCodeImpulse
from gaia_sdk.graphql.request.input.CreateEdgeImpulse import CreateEdgeImpulse
from gaia_sdk.graphql.request.input.DeleteEdgeImpulse import DeleteEdgeImpulse
from gaia_sdk.graphql.request.type.Experience import Experience as ExperienceReq
from gaia_sdk.graphql.request.type.Fulfilment import Fulfilment as FulfilmentReq
from gaia_sdk.graphql.request.type.Identity import Identity as IdentityReq
from gaia_sdk.graphql.request.type.Intent import Intent as IntentReq
from gaia_sdk.graphql.request.type.Introspection import Introspection as IntrospectionReq
from gaia_sdk.graphql.request.type.Knowledge import Knowledge as KnowledgeReq
from gaia_sdk.graphql.request.type.Edge import Edge as EdgeReq
from gaia_sdk.graphql.request.type.Perception import Perception as PerceptionReq
from gaia_sdk.graphql.request.type.Preservation import Preservation as PreservationReq
from gaia_sdk.graphql.request.type.Prompt import Prompt as PromptReq
from gaia_sdk.graphql.request.type.Query import Query as QueryReq
from gaia_sdk.graphql.request.type.Retrieval import Retrieval as RetrievalReq
from gaia_sdk.graphql.request.type.SkillIntrospection import SkillIntrospection as SkillIntrospectionReq
from gaia_sdk.graphql.request.type.Statement import Statement as StatementReq
from gaia_sdk.graphql.request.type.UpdatedIntentImpulse import UpdatedIntentImpulse
from gaia_sdk.graphql.response.type.PerceivedImpulse import PerceivedImpulse
from gaia_sdk.graphql.response.type.Query import Query as QueryRes
from gaia_sdk.graphql.response.type.Perception import Perception as PerceptionRes
from gaia_sdk.graphql.response.type.CreatedIdentityImpulse import CreatedIdentityImpulse
from gaia_sdk.graphql.response.type.UpdatedIdentityImpulse import UpdatedIdentityImpulse
from gaia_sdk.graphql.response.type.DeletedIdentityImpulse import DeletedIdentityImpulse
from gaia_sdk.graphql.response.type.CreatedIntentImpulse import CreatedIntentImpulse
from gaia_sdk.graphql.response.type.UpdatedIntentImpulse import UpdatedIntentImpulse
from gaia_sdk.graphql.response.type.DeletedIntentImpulse import DeletedIntentImpulse
from gaia_sdk.graphql.response.type.CreatedPromptImpulse import CreatedPromptImpulse
from gaia_sdk.graphql.response.type.UpdatedPromptImpulse import UpdatedPromptImpulse
from gaia_sdk.graphql.response.type.DeletedPromptImpulse import DeletedPromptImpulse
from gaia_sdk.graphql.response.type.CreatedStatementImpulse import CreatedStatementImpulse
from gaia_sdk.graphql.response.type.UpdatedStatementImpulse import UpdatedStatementImpulse
from gaia_sdk.graphql.response.type.DeletedStatementImpulse import DeletedStatementImpulse
from gaia_sdk.graphql.response.type.CreatedFulfilmentImpulse import CreatedFulfilmentImpulse
from gaia_sdk.graphql.response.type.UpdatedFulfilmentImpulse import UpdatedFulfilmentImpulse
from gaia_sdk.graphql.response.type.DeletedFulfilmentImpulse import DeletedFulfilmentImpulse
from gaia_sdk.graphql.response.type.CreatedBehaviourImpulse import CreatedBehaviourImpulse
from gaia_sdk.graphql.response.type.UpdatedBehaviourImpulse import UpdatedBehaviourImpulse
from gaia_sdk.graphql.response.type.DeletedBehaviourImpulse import DeletedBehaviourImpulse
from gaia_sdk.graphql.response.type.CreatedCodeImpulse import CreatedCodeImpulse
from gaia_sdk.graphql.response.type.UpdatedCodeImpulse import UpdatedCodeImpulse
from gaia_sdk.graphql.response.type.DeletedCodeImpulse import DeletedCodeImpulse
from gaia_sdk.graphql.response.type.CreatedEdgeImpulse import CreatedEdgeImpulse
from gaia_sdk.graphql.response.type.DeletedEdgeImpulse import DeletedEdgeImpulse
from gaia_sdk.graphql.response.type.Preservation import Preservation as PreservationRes
from gaia_sdk.graphql.response.type.SkillIntrospection import SkillIntrospection as SkillIntrospectionRes
from gaia_sdk.graphql.response.type.Introspection import Introspection as IntrospectionRes
from gaia_sdk.graphql.response.type.Behaviour import Behaviour as BehaviourRes
from gaia_sdk.graphql.response.type.Code import Code as CodeRes
from gaia_sdk.graphql.response.type.Fulfilment import Fulfilment as FulfilmentRes
from gaia_sdk.graphql.response.type.Statement import Statement as StatementRes
from gaia_sdk.graphql.response.type.Prompt import Prompt as PromptRes
from gaia_sdk.graphql.response.type.Identity import Identity as IdentityRes
from gaia_sdk.graphql.response.type.Intent import Intent as IntentRes
from gaia_sdk.graphql.response.type.Edge import Edge as EdgeRes
from gaia_sdk.graphql.response.type.Knowledge import Knowledge as KnowledgeRes
from gaia_sdk.graphql.response.type.Experience import Experience as ExperienceRes
from gaia_sdk.graphql.response.type.Retrieval import Retrieval as RetrievalRes
from gaia_sdk.graphql.request.type.Mutation import Mutation as MutationReq
from gaia_sdk.graphql.response.type.Mutation import Mutation as MutationRes
from gaia_sdk.graphql.request.type.PerceivedImpulse import PerceivedImpulse
from gaia_sdk.graphql.request.input.PerceiveActionImpulse import PerceiveActionImpulse
from gaia_sdk.graphql.request.input.PerceiveDataImpulse import PerceiveDataImpulse
from gaia_sdk.graphql.request.type.Conversational import Conversational as ConversationalReq
from gaia_sdk.graphql.response.type.Conversational import Conversational as ConversationalRes
from gaia_sdk.graphql.request.input.PerceiveDataImpulse import PerceiveDataImpulse
from gaia_sdk.graphql.request.input.PerceiveActionImpulse import PerceiveActionImpulse
from gaia_sdk.graphql.request.type.BehaviourExecution import BehaviourExecution as BehaviourExecutionReq
from gaia_sdk.graphql.request.type.BehaviourExecutionDetail import BehaviourExecutionDetail as BehaviourExecutionDetailReq
from gaia_sdk.graphql.response.type.BehaviourExecution import BehaviourExecution as BehaviourExecutionRes
from gaia_sdk.graphql.response.type.BehaviourExecutionDetail import BehaviourExecutionDetail as BehaviourExecutionDetailRes
from gaia_sdk.graphql.request.type.SkillProvisionBuildJob import SkillProvisionBuildJob as SkillProvisionBuildJobReq
from gaia_sdk.graphql.response.type.SkillProvisionBuildJob import SkillProvisionBuildJob as SkillProvisionBuildJobRes

Uuid = dict




from gaia_sdk.graphql.request.type.Conversational import Conversational
from gaia_sdk.graphql.request.type.PerceivedImpulse import PerceivedImpulse
from gaia_sdk.graphql.request.input.PerceiveDataImpulse import PerceiveDataImpulse
from gaia_sdk.graphql.request.input.PerceiveActionImpulse import PerceiveActionImpulse

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class Perception(list):
    """
    This type contains all perception sensor impulses which are used to invoke
    events in gaia.
    """

    """
    Contains all perception fields needed for a conversation.
    """
    def conversational(self, config: Callable[['Conversational'], None]):
        def callback(registry: VariableRegistry):
            entity = Conversational()
            config(entity)
            return "conversational {" + entity.render(registry) + "}"
        self.append(callback)

    """
    Data perception impulse used to invoke a data transformation behaviour
    """
    def perceive_data(self, impulse: PerceiveDataImpulse, config: Callable[['PerceivedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = PerceivedImpulse()
            config(entity)
            return f'perceiveData(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    """
    Action perception impulse used to invoke a data transformation behaviour
    """
    def perceive_action(self, impulse: PerceiveActionImpulse, config: Callable[['PerceivedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = PerceivedImpulse()
            config(entity)
            return f'perceiveAction(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))





from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class DeleteEdgeImpulse():
    """
    The specification to delete an edge instance
    """
    source: str
    edgeId: str

    def __init__(self, source: str, edgeId: str):
        self.source = source
        self.edgeId = edgeId

    def __eq__(self, other):
        if type(other) is type(self):
            return self.source == other.source and self.edgeId == other.edgeId
        return False

    def __repr__(self):
        return {'source': self.source, 'edgeId': self.edgeId}




from gaia_sdk.graphql.request.type.ConnectNodeRemovedImpulse import ConnectNodeRemovedImpulse
from gaia_sdk.graphql.request.type.ConnectNodeUnsetImpulse import ConnectNodeUnsetImpulse
from gaia_sdk.graphql.request.type.ConnectNodeAppendedImpulse import ConnectNodeAppendedImpulse
from gaia_sdk.graphql.request.type.ConnectNodeSetImpulse import ConnectNodeSetImpulse
from gaia_sdk.graphql.request.input.ConnectSetNodeImpulse import ConnectSetNodeImpulse
from gaia_sdk.graphql.request.input.ConnectAppendNodeImpulse import ConnectAppendNodeImpulse
from gaia_sdk.graphql.request.input.ConnectUnsetNodeImpulse import ConnectUnsetNodeImpulse
from gaia_sdk.graphql.request.input.ConnectRemoveNodeImpulse import ConnectRemoveNodeImpulse

from typing import Callable, List
from gaia_sdk.api.VariableRegistry import VariableRegistry
from gaia_sdk.graphql.request.enumeration.Order import Order
from gaia_sdk.graphql.request.enumeration.OrderByField import OrderByField
from gaia_sdk.graphql.request.enumeration.EdgeOrderByField import EdgeOrderByField
from gaia_sdk.graphql.request.enumeration.EdgeType import EdgeType


class ConnectNodeKnowledge(list):

    def append(self, impulse: ConnectAppendNodeImpulse, config: Callable[['ConnectNodeAppendedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = ConnectNodeAppendedImpulse()
            config(entity)
            return f'append(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def remove(self, impulse: ConnectRemoveNodeImpulse, config: Callable[['ConnectNodeRemovedImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = ConnectNodeRemovedImpulse()
            config(entity)
            return f'remove(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def set(self, impulse: ConnectSetNodeImpulse, config: Callable[['ConnectNodeSetImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = ConnectNodeSetImpulse()
            config(entity)
            return f'set(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def unset(self, impulse: ConnectUnsetNodeImpulse, config: Callable[['ConnectNodeUnsetImpulse'], None]):
        def callback(registry: VariableRegistry):
            name1 = registry.register("impulse", impulse)
            entity = ConnectNodeUnsetImpulse()
            config(entity)
            return f'unset(impulse:{name1})' + '{' + entity.render(registry) + '}'
        self.append(callback)

    def render(self, registry: VariableRegistry):
        return " ".join(map(lambda e: e(registry), self))
